%Clean script

if ~isdeployed
    clearvars
    close('all')
end

%% User Inputs

%Choose models to run, by grouping: linear, net increase 3 year, and
%alternative net increase 3 year.
%
% Constructing model name:
%  var1_var2_var3_modelType (any combination of valid variable types, plus a model type at the end)
%  where variables may be: 
%   gdp: log real US GDP
%   oil: log US imported refiner acquisition cost, deflated by US CPI
%   bp: bond premium, from Gilchrist & Zagrajsek (2012)
%    bp1 - converted to quarterly by first of period
%    bp3 - converted to quarterly by last of period
%    bpavg - converted to quarterly by period average
%   tr: treasury spread
%    tr1 - converted to quarterly by first of period
%    tr3 - converted to quarterly by last of period
%    travg - converted to quarterly by period average
%   mic: Michigan consumer confidence
% and modelType may be:
%   lin: linear
%   ni3: 3-year net increase measure, where net increase is applied to
%    first variable in system (so make sure first variable is oil). net
%    increase measure affects all variables.
%   altni3: 3-year net increase measure, where net increase is applied to
%    first variable in system (so make sure first variable is oil). net
%    increase measure affects only second variable (which is almost always GDP).
%  **Note, if no model type is given, model type is set to linear

%Group models into linmodels, ni3models, and altmodels - important for
%proper functionality

% linmodels = {'gdp_lin','oil_gdp','gdp_bp1','gdp_travg','gdp_mic','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3'};

% linmodels = {'gdp_lin','oil_gdp','gdp_bp1','gdp_travg','gdp_mic','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3'};

% linmodels = {'oil_gdp','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_bp1_ni3','oil_gdp_travg_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3'};

linmodels = {'oil_gdp'};
ni3models = {'oil_gdp_ni3'};
altmodels = [];

allmodels = [linmodels ni3models altmodels];

%Now - select models to run as data generating processes and to fit on top
%of data generating processes. All of these models should also appear in
%allmodels variable set up above, as we want to have the empirical values
%as well.
dgpmodels = {'oil_gdp_ni3'};
fitmodels = {'oil_gdp','oil_gdp_ni3'};

% Set parameters for VARs and Impulse Responses

%Number of lags to include in VAR estimation
nlags = 4;
%If hascon = 1, include constant in regression.
hascon = 1;
%For varsim, varsim_errfeed, and new_nlimpkv functions, ndraws is the number 
%of simulations to run inside function. Is the number of simulations over
%which to take mean forecast/impulse response for non-linear model, and the
%number of simulations over which to calculate variance for all model
%types.
ndraws = 50000;
%Number of steps, starting from 0 to nstep-1, ahead to calculate forecasts
%and impulse responses
nstep = 9;
%if monte = 0, draw from empirical distribution. If monte = 1, draw from 
%normal distribution, scaled to variance of model errors.
monte = 0;
%shkvalue - first entry has no meaning, second through last entries are
%number of standard deviations of shock to impose. Note, section 8 where
%wald statistics are calculated assumes shkvalue = [1 -2 -1 1 2]
shkvalue = [1 -2 -1 1 2];
%Set the random number generator
useRngSeed = 35400;
rng(useRngSeed)

shk_numval = [-.32 -.16 .16 .32]; %Use these rather than shkvalue below for empval series
nsimdataset = 100000;

%Date range string (e.g. 1980Q1:1999Q4) for period to use in differenced
%data in VAR
%to use full sample, set to []
setEstimationPeriod = [];

%Choose infosetdates (i.e. Omega_{t-1}) to run. A given infosetdate is the
%last period of lagged values used in calculating a forecast/impulse
%response/historical decomposition/variance decomposition.
%To use all available infosetdates , set to []
%
% Note - infosetdate must be a date in the range selected by
% setEstimationPeriod, given current set up of code. It is feasible,
% however, to change the code so that betas are estimated on one set of
% dates, and impulse responses/historical decompositions/variance
% decompositions still have the full set of dates available to them.

% infosetdates = {'1979Q1','1979Q4','1980Q4','1981Q2','1990Q2','1990Q3','1994Q3',...
%     '1995Q4','1996Q3','1999Q4','2001Q1','2004Q2','2004Q4','2007Q2','2007Q4','2008Q4'};

infosetdates = {'2008Q4'};

outFileName = 'outputOfCleanScript_WaldSim_empval_2008Q4';
outFileName = [outFileName '_' lower(datestr(now,'ddmmmyyyy'))];
savedata = 1;

%for speed, toggle whether to calculate impulse responses or not, and
%whether to calculate historical decompositions/variance decompositions or
%not
calc_impulseresponse = 1;
calc_histAndVarDecomp = 1;
calc_waldstats = 0;

disp('Started script')

nshocks = length(shkvalue)-1;

%% Section 1: Load in data
%
% 1. Load in data for oil, gdp, and Bond Premiums. Convert to desired
% values - for oil, calculate real oil log and log difference, converting to
% quarterly. For GDP, real gdp log and log difference, and for bond premium
% convert level to quarterly. Oil and Bond premium can be converted by
% beginning or end of period (see cnvrtbase_oil and cnvrtbase_bp)
%
% 2. Merge series by date. GDP on its own runs (in levels) from 1947Q1 to
% 2012Q2. Oil on its own runs (in levels) from Jan1974 (1974Q1) to Aug2012 
% (2012Q2). Bond Premiums run from Jan1971 (1971Q1) to Sep2010 (2010Q3).
% Also create a date vector/date string (Matlab Formats) for each of these.
%
% 3. Using the series from 1974Q1 to 2012Q2, set data, data level, data
% plus (i.e. including non-linear oil terms), beta estimates, VAR in-sample
% errors, set oilindex and crindex (which column of data corresponds to
% oil and cr, respectively), and estimation options (to be used by varsim
% and releated programs) for a variety of models: oil_gdp, gdp_bp,
% oil_gdp_bp, oil_gdp_ni3, oil_gdp_bp_ni3, and oil_gdp_bp_altni3.

%Set conversion basis for monthly to quarterly: beginning or end
cnvrtbase_oil = 'end';
cnvrtbase_bp = 'beg';

if strcmpi(cnvrtbase_oil,'beg')
    baseper_oil = 1;
elseif strcmpi(cnvrtbase_oil,'end')
    baseper_oil = 3;
end

if strcmpi(cnvrtbase_bp,'beg')
    baseper_bp = 1;
elseif strcmpi(cnvrtbase_bp,'end')
    baseper_bp = 3;
end

if ~isdeployed
    restoredefaultpath
    addpath('T:\TQS\Vigfusson\matlabcommon')
    addpath('T:\TQS\Vigfusson\matlabcommon\varcode\')
    addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\')
    addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\tvresp')
    addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\tvresp\creditspread')
end

oildatafile = 'mnt_racandcpi_12202013.csv';
gdpdatafile = 'qtr_rgdp_12202013.csv';

%Prepare real oil prices series
[data,varnames,rawdates] = tblread(oildatafile,'comma'); %#ok<ASGLU>
poilm_date = [rawdates(:,1:3) repmat('19',[size(rawdates,1) 1]) rawdates(:,end-1:end)];
tmpindex = find(strcmpi(cellstr(rawdates(:,end-1:end)),'00'),1,'first');
tmplen = length(poilm_date(tmpindex:end,:));
poilm_date(tmpindex:end,end-3:end-2) = repmat('20',[tmplen 1]);
poilm_date = cellstr(poilm_date);

poilm = data(:,1);
poilm_cpi = data(:,2);

%This data will start in Jan1974 and end in Aug2012
tmpindex = ~isnan(poilm);
poilm = log(poilm(tmpindex));
poilm_cpi = log(poilm_cpi(tmpindex));
poilm_date = poilm_date(tmpindex);

%Convert oil from monthly to quarterly taking period-end value
%Data will be 1974Q1 to 2012Q2
poilq = poilm(baseper_oil:3:end);
poilq_cpi = poilm_cpi(baseper_oil:3:end);
poilq_date = cellstr(datestr(datenum(poilm_date(baseper_oil:3:end),'mmmyyyy'),'yyyyqq'));

levrpoil = poilq-poilq_cpi;
levrpoil_date = poilq_date;

levnompoil = poilq;
dnompoil = diff(poilq);

drpoil = diff(poilq) - diff(poilq_cpi);
drpoil_date = poilq_date(2:end);

disp(' ')
disp('Oil data in')

%Prepare real gdp series
[data,varnames,rawdates] = tblread(gdpdatafile,'comma'); %#ok<ASGLU>

if strcmpi(gdpdatafile,'qlutzv2012m10.csv')
    levrgdp_date = [repmat('19',[length(rawdates) 1]) rawdates(:,1:2) repmat('Q',[length(rawdates) 1]) rawdates(:,end)];
    tmpindex = find(strcmpi(cellstr(rawdates(:,1:2)),'00'),1,'first');
    tmplen = length(rawdates(tmpindex:end,1));
    levrgdp_date(tmpindex:end,1:2) = repmat('20',[tmplen 1]);
    levrgdp_date = cellstr(levrgdp_date);
    
    levrgdp = log(data(:,2));
    tmpindex = ~isnan(levrgdp);
    levrgdp = levrgdp(tmpindex);
    levrgdp_date = levrgdp_date(tmpindex);
    
elseif strcmpi(gdpdatafile,'qtr_rgdp_12202013.csv')
    levrgdp_date = cellstr(rawdates);
    levrgdp = log(data(:,1));
    tmpindex = ~isnan(levrgdp);
    levrgdp = levrgdp(tmpindex);
    levrgdp_date = levrgdp_date(tmpindex);
end

disp(' ')
disp('GDP data in')

%This data will start in 1947Q1 and end in 2012Q2

drgdp = diff(levrgdp);
drgdp_date = levrgdp_date(2:end);

%Prepare Bond Premium series
[data,varnames,rawdates] = tblread('GZ_M_updated.csv','comma');
%Data starts in JAN1973 and ends in DEC2012
bondpremm_date = cellstr(datestr(datenum(rawdates,'mmm-yy'),'mmmyyyy'));

bondpremm = data(:,2);

tmpindex = ~isnan(bondpremm);
bondpremm = bondpremm(tmpindex);
bondpremm_date = bondpremm_date(tmpindex);

%This data will run from 1973Q1 to 2012Q4
bondpremq = bondpremm(baseper_bp:3:end);
bondpremq_1 = bondpremm(1:3:end);
bondpremq_3 = bondpremm(3:3:end);
bondpremq_avg = (1/3)*(bondpremm(1:3:end)+bondpremm(2:3:end)+bondpremm(3:3:end));
bondpremq_date = unique(cellstr(datestr(datenum(bondpremm_date(baseper_bp:3:end),'mmmyyyy'),'yyyyqq')));

disp(' ')
disp('Bond data in')

[data,varnames,rawdates] = tblread('treasyield.csv','comma');

treasm_date = cellstr(rawdates);
treasq_date = unique(cellstr(datestr(datenum(treasm_date,'mmmyyyy'),'yyyyqq')));

treasspreadm = data(:,1) - data(:,2); %10 year minus 5 year;
treasspreadq_1 = treasspreadm(1:3:end);
treasspreadq_3 = treasspreadm(3:3:end);
treasspreadq_avg = (1/3)*(treasspreadm(1:3:end)+treasspreadm(2:3:end)+treasspreadm(3:3:end));

disp(' ')
disp('Treas data in')

[data,varnames,qtrs] = tblread('survey.csv','comma');

michiconfidence_date = cellstr([num2str(data(2:end,1)) repmat('Q',[size(data,1)-1 1]) qtrs(2:end,1)]);
michiconfidence = data(2:end,2);
michiconfidence_log = log(michiconfidence);
michiconfidence_log_date = michiconfidence_date;
michiconfidence_dlog = diff(michiconfidence_log);
michiconfidence_dlog_date = michiconfidence_date(2:end);

disp(' ')
disp('Mich data in')

%Get dates where all data is available (1974Q2 to 2012Q2)
effdate = intersect(intersect(drgdp_date,drpoil_date,'stable'),bondpremq_date,'stable');
leveffdate = intersect(intersect(levrgdp_date,levrpoil_date,'stable'),bondpremq_date,'stable');

effdate = intersect(effdate,treasq_date,'stable');
leveffdate = intersect(leveffdate,treasq_date,'stable');

effdate = intersect(effdate,michiconfidence_date,'stable');
leveffdate = intersect(leveffdate,michiconfidence_date,'stable');

[~,drpoilindex] = intersect(drpoil_date,effdate,'stable');
[~,drgdpindex] = intersect(drgdp_date,effdate,'stable');
[~,bpindex] = intersect(bondpremq_date,effdate,'stable');
[~,trindex] = intersect(treasq_date,effdate,'stable');
[~,micindex] = intersect(michiconfidence_date,effdate,'stable');
[~,levrpoilindex] = intersect(levrpoil_date,leveffdate,'stable');
[~,levrgdpindex] = intersect(levrgdp_date,leveffdate,'stable');
[~,levbpindex] = intersect(bondpremq_date,leveffdate,'stable');
[~,levtrindex] = intersect(treasq_date,leveffdate,'stable');
[~,levmicindex] = intersect(michiconfidence_date,leveffdate,'stable');

drpoil_reg = drpoil(drpoilindex);
levrpoil_reg = levrpoil(levrpoilindex);

dnompoil_reg = dnompoil(drpoilindex);
levnompoil_reg = levnompoil(levrpoilindex);

drgdp_reg = drgdp(drgdpindex);
levrgdp_reg = levrgdp(levrgdpindex);

bondpremq_1_reg = bondpremq_1(bpindex);
levbondpremq_1_reg = bondpremq_1(levbpindex);

bondpremq_3_reg = bondpremq_3(bpindex);
levbondpremq_3_reg = bondpremq_3(levbpindex);

bondpremq_avg_reg = bondpremq_avg(bpindex);
levbondpremq_avg_reg = bondpremq_avg(levbpindex);

treasspreadq_1_reg = treasspreadq_1(trindex);
levtreasspreadq_1_reg = treasspreadq_1(levtrindex);

treasspreadq_3_reg = treasspreadq_3(trindex);
levtreasspreadq_3_reg = treasspreadq_3(levtrindex);

treasspreadq_avg_reg = treasspreadq_avg(trindex);
levtreasspreadq_avg_reg = treasspreadq_avg(levtrindex);

michiconfidence_reg = michiconfidence(micindex);
levmichiconfidence_reg = michiconfidence(levmicindex);

%Start structures for holding output and parameters for each model
%Begin by identifying the order of variables (gdp, oil, bond premium) in
%the columns of data. If a variable is not included, give empty array.

allvariablematrix = [drpoil_reg, dnompoil_reg, drgdp_reg,...
                    bondpremq_1_reg, bondpremq_3_reg, bondpremq_avg_reg,...
                    treasspreadq_1_reg, treasspreadq_3_reg, treasspreadq_avg_reg,...
                    michiconfidence_reg];
allvariablematrix_lev = [levrpoil_reg, levnompoil_reg, levrgdp_reg,...
                    levbondpremq_1_reg, levbondpremq_3_reg, levbondpremq_avg_reg,...
                    levtreasspreadq_1_reg, levtreasspreadq_3_reg, levtreasspreadq_avg_reg,...
                    levmichiconfidence_reg];                
                
allvariablestubs = {'oil','nomoil','gdp','bp1','bp3','bpavg','tr1','tr3','travg','mic'};
oilvariablestubs = {'oil','nomoil'};
bpvariablestubs = {'bp1','bp3','bpavg'};
trvariablestubs = {'tr1','tr3','travg'};
allmodelstubs = {'lin','ni3','altni3'};

%% Section 2: Organize data for models to run
%Prepare data and level for each of the selected models, by selecting
%appropriate columns of allvariablematrix and allvariablematrix_lev. Then
%assign variable column index value (e.g. gdpix, oilix) fields for the
%each model. If a variable is not included in the model, then its ix field
%is set to []
%
%Fields assigned for datastruct.[model name]
%data, lev, gdpix, oilix, bpix, trix, micix

for imodel = 1:length(allmodels)
    
    usemodel = allmodels{imodel};
    
    score_index = strfind(usemodel,'_');
    storevars = cell(length(score_index)+1,1);
    
    for ivar = 1:length(score_index)+1;
        if ivar == 1
            storevars{ivar} = usemodel(1:score_index(1)-1);
        elseif ivar == length(score_index)+1
            storevars{ivar} = usemodel(score_index(ivar-1)+1:end);
        else
            storevars{ivar} = usemodel(score_index(ivar-1)+1:score_index(ivar)-1);
        end
    end
    
    if any(~ismember(storevars,union(allvariablestubs,allmodelstubs)))
        error('Invald input string for model')
    end
    
    storevars = storevars(~ismember(storevars,allmodelstubs));    
    
    if any(strcmpi(storevars,'gdp'))
       if ~isscalar(find(strcmpi(storevars,'gdp'),1))
           error('Invalid input string for model - more than one gdp input.')
       else        
            empstruct.(usemodel).gdpix = find(strcmpi(storevars,'gdp'),1);
       end
    else
        empstruct.(usemodel).gdpix = [];
    end
    
    if any(ismember(storevars,oilvariablestubs))
       if ~isscalar(find(ismember(storevars,oilvariablestubs),1))
           error('Invalid input string for model - more than one oil input.')
       else        
            empstruct.(usemodel).oilix = find(ismember(storevars,oilvariablestubs),1);
       end
    else
        empstruct.(usemodel).oilix = [];
    end
    
    if any(ismember(storevars,bpvariablestubs))
       if ~isscalar(find(ismember(storevars,bpvariablestubs)))
           error('Invalid input string for model - more than one bond premium variable.')
       else
           empstruct.(usemodel).bpix = find(ismember(storevars,bpvariablestubs));
       end
    else
        empstruct.(usemodel).bpix = [];
    end
    
    if any(ismember(storevars,trvariablestubs))
       if ~isscalar(find(ismember(storevars,trvariablestubs)))
           error('Invalid input string for model - more than one bond premium variable.')
       else
           empstruct.(usemodel).trix = find(ismember(storevars,trvariablestubs));
       end
    else
        empstruct.(usemodel).trix = [];
    end
    
    if any(strcmpi(storevars,'mic'))
        if ~isscalar(find(strcmpi(storevars,'mic'),1))
           error('Invalid input string for model - more than one gdp input.')
       else        
            empstruct.(usemodel).micix = find(strcmpi(storevars,'mic'),1);
       end
    else
        empstruct.(usemodel).micix = [];
    end
    
    mapdatamatrix = zeros(length(allvariablestubs),length(storevars));
    
    for ivar = 1:length(storevars);
        mapdatamatrix(strcmpi(storevars(ivar),allvariablestubs),ivar) = 1;
    end
    
    empstruct.(usemodel).data = allvariablematrix*mapdatamatrix;
    empstruct.(usemodel).lev = allvariablematrix_lev*mapdatamatrix;
    
end

%% Section 3: select estimation period, and check infosetdates are valid
%
%Given input for setEstimationPeriod, find the indecies of effdate and
%leveffdate (all dates where we have all the data we loaded in)
%corresponding to setEstimationPeriod. Then check that infosetdates are in
%this sample period, conditional on number of lags for models.
%
%Note, as currently written, the length of each simulated data set for a
%dgp will be the length of the full available sample, NOT the length of the
%sample selected for here.
%

if isempty(setEstimationPeriod);
    useEstimationPeriod_index = 1:length(effdate);
    useEstimationPeriodLev_index = 1:length(leveffdate);
else
    colon_index = strfind(setEstimationPeriod,':');
    startEstPeriod = setEstimationPeriod(1:colon_index-1);
    endEstPeriod = setEstimationPeriod(colon_index+1:end);
    startEstPeriod_index = find(strcmpi(startEstPeriod,effdate)==1);
    endEstPeriod_index = find(strcmpi(endEstPeriod,effdate)==1);    
    useEstimationPeriod_index = startEstPeriod_index:endEstPeriod_index;
    startEstPeriodLev_index = find(strcmpi(startEstPeriod,leveffdate)==1)-1;
    endEstPeriodLev_index = find(strcmpi(endEstPeriod,leveffdate)==1);
    useEstimationPeriodLev_index = startEstPeriodLev_index:endEstPeriodLev_index;    
end

if isempty(useEstimationPeriod_index)
    error('Input dates for setEstimationPeriod not found in effdate.')
end

if isempty(useEstimationPeriodLev_index)
    error('Input dates for setEstimationPeriod not found in effdate.')
end

%Use nlags and effdate to determine number and dates of all possible
%startpoints. Convention for naming startpoints is the first forecast
%period based on that start point. infosetdates (i.e. Omega_{t-1}) is the
%last period of lags 

allstartpoints = effdate(nlags+1:end); %Date of first forecast period of all startpoints

%For historical decomposition, we find regression errors depending on what
%we used to calculate betas - so need to find the subset of startpoints
%that we will have errors for.
effdate_estsubset = effdate(useEstimationPeriod_index);
allstartpoints_estsubset = effdate_estsubset(nlags+1:end);

%Technically, last start point would forecast 1st period outside the 
%sample. Add this to allstartpoints.
[tmp_yr,tmp_mnt] = datevec(allstartpoints{end},'yyyyqq');
allstartpoints = [allstartpoints; cellstr(datestr(datenum(tmp_yr,tmp_mnt+3,1),'yyyyqq'))];

[tmp_yr,tmp_mnt] = datevec(allstartpoints_estsubset{end},'yyyyqq');
allstartpoints_estsubset = [allstartpoints_estsubset; cellstr(datestr(datenum(tmp_yr,tmp_mnt+3,1),'yyyyqq'))];

%Infosetdates are one quarter prior to start points
allinfosetdates = cell(length(allstartpoints),1);
for idate = 1:length(allstartpoints)
    allinfosetdates(idate) = cellstr(datestr(addtodate(datenum(allstartpoints(idate),'yyyyqq'),-3,'month'),'yyyyqq'));
end

allinfosetdates_estsubset = cell(length(allstartpoints_estsubset),1);
for idate = 1:length(allstartpoints_estsubset)
    allinfosetdates_estsubset(idate) = cellstr(datestr(addtodate(datenum(allstartpoints_estsubset(idate),'yyyyqq'),-3,'month'),'yyyyqq'));
end

%If infosetdates is empty, use all the legal infosetdates
if isempty(infosetdates)
    infosetdates = allinfosetdates_estsubset;
else
    
    %otherwise, check that input infosetdates are legal given estimation
    %period
    if any(~ismember(infosetdates,allinfosetdates_estsubset))
        error('One or more infosetdates supplied by user is not available given the input data and setting for setEstimationPeriod')
    end
end

%Generate list of start points based on list of infosetdates provided above
showstartpoints = cell(1,length(infosetdates));
for idate = 1:length(infosetdates)
    showstartpoints{idate} = datestr(addtodate(datenum(infosetdates{idate},'yyyyqq'),3,'month'),'yyyyqq');
end

nstartpoints = length(showstartpoints);

%% Section 4: Estimate beta for model, given estimation period
%
%For linear, 3-year net increase, and alternate 3-year net increase models,
%use modelinput function to find options for vhist, betaopt, and simopt.
%Given these option settings, use new_estimatemodel function to calculate
%dataplus (i.e. data with non-linear term appended as additional column),
%beta estimate, and regression errors.
%
%Fields for datastruct.[model name] (e.g. datastruct.oil_gdp) filled in:
%vhist, betaopt, simopt, beta, erz, dataplus
%

%Run VAR for linear models, and store results/options in structure
[vhist_lin,betaopt_lin,simopt_lin,exog_lin,restr_lin,impstr_lin] = modelinput('lin_cu','impulse'); %modelinput: lin_cu --> linear model, with leads for cholesky decomposition

for imodel = 1:length(linmodels)
    
    empstruct.(linmodels{imodel}).vhist = vhist_lin;
    empstruct.(linmodels{imodel}).betaopt = betaopt_lin;    
    empstruct.(linmodels{imodel}).simopt = simopt_lin;
    empstruct.(linmodels{imodel}).exog = exog_lin;
    empstruct.(linmodels{imodel}).restr = restr_lin;
    empstruct.(linmodels{imodel}).impstr = impstr_lin;    
    
    [beta_lin,erz_bp,dataplus_lin] = new_estimatemodel(empstruct.(linmodels{imodel}).data(useEstimationPeriod_index,:),empstruct.(linmodels{imodel}).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_lin,betaopt_lin);
    
    empstruct.(linmodels{imodel}).beta = beta_lin;
    empstruct.(linmodels{imodel}).erz = erz_bp;
    empstruct.(linmodels{imodel}).dataplus = dataplus_lin;    

end

%Run VAR for Net Increase model, and store results/options in structure
[vhist_ni3,betaopt_ni3,simopt_ni3,exog_ni3,restr_ni3,impstr_ni3] = modelinput('ham3_cu','impulse'); %ham3_cu --> 3-year net increase model, with leads for cholesky decomposition

for imodel = 1:length(ni3models)
    
    empstruct.(ni3models{imodel}).vhist = vhist_ni3;
    empstruct.(ni3models{imodel}).betaopt = betaopt_ni3;    
    empstruct.(ni3models{imodel}).simopt = simopt_ni3;
    empstruct.(ni3models{imodel}).exog = exog_ni3;
    empstruct.(ni3models{imodel}).restr = restr_ni3;
    empstruct.(ni3models{imodel}).impstr = impstr_ni3;    
    
    [beta_ni3,erz_ni3,dataplus_ni3] = new_estimatemodel(empstruct.(ni3models{imodel}).data(useEstimationPeriod_index,:),empstruct.(ni3models{imodel}).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_ni3,betaopt_ni3);
    
    empstruct.(ni3models{imodel}).beta = beta_ni3;
    empstruct.(ni3models{imodel}).erz = erz_ni3;
    empstruct.(ni3models{imodel}).dataplus = dataplus_ni3;    

end

%Construct beta and erz for Alternative Net Increase model by using results
%from oil_gdp_bp_ni3 and oil_gdp_bp calculated above

for imodel = 1:length(altmodels)
    
    usemodel_alt = altmodels{imodel};
    score_index = strfind(usemodel_alt,'_');
    usemodel_lin = usemodel_alt(1:score_index(end)-1);
    usemodel_ni3 = [usemodel_lin '_ni3'];
    
    empstruct.(usemodel_alt).vhist = vhist_ni3;
    empstruct.(usemodel_alt).betaopt = betaopt_ni3;
    empstruct.(usemodel_alt).simopt = simopt_ni3;
    
    if ismember(usemodel_lin,fields(empstruct))
        usebeta_lin = empstruct.(usemodel_lin).beta;
        useerz_lin = empstruct.(usemodel_lin).erz;
    else
        [usebeta_lin,useerz_lin] = new_estimatemodel(empstruct.(usemodel_alt).data(useEstimationPeriod_index,:),empstruct.(usemodel_alt).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_lin,betaopt_lin);
    end
    
    if ismember(usemodel_ni3,fields(empstruct))
        usebeta_ni3 = empstruct.(usemodel_ni3).beta;
        useerz_ni3 = empstruct.(usemodel_ni3).erz;
        usedataplus_ni3 = empstruct.(usemodel_ni3).dataplus;        
    else
        [usebeta_ni3,useerz_ni3,usedataplus_ni3] = new_estimatemodel(empstruct.(usemodel_alt).data(useEstimationPeriod_index,:),empstruct.(usemodel_alt).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_ni3,betaopt_ni3);        
    end

    %Alternative errors are same as linear for Bond Premium, and net increase
    %for others
    erz_alt_tmp = useerz_ni3;
    erz_alt_tmp(:,3:end) = useerz_lin(:,3:end);    

    %Coefficient estimates are same as linear for Bond Premium, and net increase
    %for others. Linear coefficients must be rearranged, and zeros placed for
    %net increase terms.
    
    nvars = size(usebeta_ni3,1);
    nvars_exoil_exgdp = nvars-2;
    
    beta_alt_tmp = NaN(size(usebeta_ni3));
    beta_alt_tmp([1 2],:) = usebeta_ni3([1 2],:); %All coefficients in oil and gdp rows
    beta_alt_tmp(3:end,1:1+nvars*nlags) = usebeta_lin(3:end,1:1+nvars*nlags); %Coefficients for constant and lags of dependents
    beta_alt_tmp(3:end,1+nvars*nlags+(1:nlags)) = zeros(nvars_exoil_exgdp,nlags); %Net increase lags get zero coefficient
    beta_alt_tmp(3:end,1+(nvars+1)*nlags+1) = usebeta_lin(3:end,1+nvars*nlags+1); %Lead oil term
    beta_alt_tmp(3:end,1+(nvars+1)*nlags+2) = zeros(nvars_exoil_exgdp,1); %Lead for net increase gets zero coefficient
    beta_alt_tmp(3:end,1+(nvars+1)*nlags+(3:nvars)) = usebeta_lin(3:end,1+nvars*nlags+(2:nvars-1)); %Lead gdp and other variable terms

    empstruct.(usemodel_alt).beta = beta_alt_tmp;
    empstruct.(usemodel_alt).erz = erz_alt_tmp;
    empstruct.(usemodel_alt).dataplus = usedataplus_ni3;
    
end

%% Section 5: Generate simulated data sets for dgp models
% Use varsim to generate a nsimdataset series of forecasts with random
% errors (drawn from the distribution determined by the setting of the
% monte option). Save this as the first field of simstruct
%
% Fields generated:
%  simstruct.[dgp model].simdata, simstruct.[dgp model].simlev
%
%  e.g. simstruct.oil_gdp.simdata

for idgp = 1:length(dgpmodels)
    
    usesimmodel = dgpmodels{idgp};
    
    if ismember(usesimmodel,linmodels)
        usesimopt = empstruct.(usesimmodel).simopt;
        usesimopt(end) = -2;
    else
        usesimopt = empstruct.(usesimmodel).simopt;
    end
    
    usedata = empstruct.(usesimmodel).data(useEstimationPeriod_index,:);
    uselev = empstruct.(usesimmodel).lev(useEstimationPeriodLev_index,:);
    [usebeta, useerz, usedataplus] = new_estimatemodel(usedata,uselev,nlags,hascon,[],empstruct.(usesimmodel).vhist,empstruct.(usesimmodel).betaopt);
    
    laststpoint = length(usedata)-nlags+1;    
    
    tmpsimdata = varsim(usedataplus,uselev,usebeta,useerz,nlags,hascon,nsimdataset,length(usedata)-nlags,monte,1,empstruct.(usesimmodel).vhist,usesimopt);
    %All series will include first nlags of actual data
    simstruct.(usesimmodel).simdata = tmpsimdata;
    %Pad in first observation of level, and add this to cumsum of differences
    simstruct.(usesimmodel).simlev = [repmat(empstruct.(usesimmodel).lev(1,:),[1 1 nsimdataset]);bsxfun(@plus,empstruct.(usesimmodel).lev(1,:),cumsum(tmpsimdata,1))];
    
end

disp(' ')
disp('Done preparing data for simulated DGP data sets.')

%% Section 6: Calculate fit model impulse response for all DGP simulates series
% For each DGP, there are nsimdataset histories we generated. Use these to
% in turn generate nsimdataset betas for each fit model. Now, for each of
% these nsimdataset betas for the given fit model, estimate the impulse
% response for the desired histories in the actual data.
%
% In other words, conditional on a given Omega_{t-1} - i.e. lagged values
% in the actual data for a given period, t, - and the parameter estimates
% beta_{i,f,d} - i.e. the betas for fit model F1 estimated on the i-th simulated
% data set from dgp model D1 - and finally a shock of N standard deviations
% - we have I_{i,f,d,s,t}, the impulse response for model F1, conditional 
% on data set i of dgp model D1 and lagged values for t (i.e. Omega_{t-1}).
%
% Note the naming convention - infosetdate of 1990Q3 means the lagged
% values used for estimating the impulse response end in 1990Q3.
%
% Note - to save time, we only have the code below set up to calculate oil
% impulse responses. We store forecasts as this is an intermediate step in
% calculating the impulse response.
%
% In addition, we use a simpler function for calculating the linear VARs,
% as these are time-invariant (i.e. change depending on f, d, and i but not
% t).
%
% Also, to save time, we use ndraws/2 paths to get the mean response
% for each i DGP data set. Below, when we estimate the impulse response
% using the beta for the actual data, we take the mean over ndraws paths.
%

for idgp = 1:length(dgpmodels)
    
    dgpmodel_use = dgpmodels{idgp};    
    
    for ifit = 1:length(fitmodels)
    
        fitmodel_use = fitmodels{ifit};
        
        fit_nvars = size(empstruct.(fitmodel_use).data,2);
        fit_dataplus = empstruct.(fitmodel_use).dataplus;
        fit_lev = empstruct.(fitmodel_use).lev;
        fit_vhist = empstruct.(fitmodel_use).vhist;
        fit_exog = empstruct.(fitmodel_use).exog;
        fit_restr = empstruct.(fitmodel_use).restr;
        fit_impstr = empstruct.(fitmodel_use).impstr;
        fit_betaopt = empstruct.(fitmodel_use).betaopt;
        fit_simopt = empstruct.(fitmodel_use).simopt;
        fit_oilix = empstruct.(fitmodel_use).oilix;
        
        fit_signew = cov(empstruct.(fitmodel_use).erz);
        fit_facsignew = chol(fit_signew)';        
        
        simstruct.(dgpmodel_use).(fitmodel_use).beta = NaN([size(empstruct.(fitmodel_use).beta),nsimdataset]);
        simstruct.(dgpmodel_use).(fitmodel_use).erz = NaN(length(effdate)-nlags,fit_nvars,nsimdataset);
        simstruct.(dgpmodel_use).(fitmodel_use).dataplus = NaN(length(effdate),size(empstruct.(fitmodel_use).dataplus,2),nsimdataset);
        simstruct.(dgpmodel_use).(fitmodel_use).frcst = NaN(nstep,fit_nvars,length(shkvalue)-1,nstartpoints,nsimdataset);
        simstruct.(dgpmodel_use).(fitmodel_use).cumfrcst = NaN(nstep,fit_nvars,length(shkvalue)-1,nstartpoints,nsimdataset);        
        simstruct.(dgpmodel_use).(fitmodel_use).oilimpz = NaN(nstep,fit_nvars,length(shkvalue)-1,nstartpoints,nsimdataset);
        simstruct.(dgpmodel_use).(fitmodel_use).cumoilimpz = NaN(nstep,fit_nvars,length(shkvalue)-1,nstartpoints,nsimdataset);
        
        for isim = 1:nsimdataset
            
            [tmpbeta,tmperz,tmpdataplus] = new_estimatemodel(simstruct.(dgpmodel_use).simdata(:,:,isim),simstruct.(dgpmodel_use).simlev(:,:,isim),nlags,hascon,[],fit_vhist,fit_betaopt);
            simstruct.(dgpmodel_use).(fitmodel_use).beta(:,:,isim) = tmpbeta;
            simstruct.(dgpmodel_use).(fitmodel_use).erz(:,:,isim) = tmperz;
            simstruct.(dgpmodel_use).(fitmodel_use).dataplus(:,:,isim) = tmpdataplus;
        
            if fit_betaopt(6)==-1 %Linear model
                [beta,sigma] = estimatevar(simstruct.(dgpmodel_use).simdata(:,:,isim),nlags,hascon);
                facsignew = chol(sigma)';
                a0 = inv(chol(sigma)');
                oilimpz = mkimprep(beta,a0,nlags,fit_oilix,nstep);
                for ishock = 1:length(shkvalue)-1
                    adjufactor = shk_numval(ishock)/facsignew(fit_oilix,fit_oilix);
                    simstruct.(dgpmodel_use).(fitmodel_use).oilimpz(:,:,ishock,:,isim) = repmat(oilimpz*adjufactor,[1 1 nstartpoints]);
                end

            else
                nstpt_use = nstartpoints;
                showstpt_use = showstartpoints;      
        
                for istpt = 1:nstpt_use
            
                    usestpt = showstpt_use{istpt};
                    usestpt_num = find(strcmpi(usestpt,allstartpoints));
                    
                    useerz = simstruct.(dgpmodel_use).(fitmodel_use).erz(:,:,isim);
                    facsignew = chol(cov(useerz))';
                    useshock = [empstruct.(fitmodel_use).oilix shk_numval/facsignew(fit_oilix,fit_oilix)];
                    
                    [~,~,~,allimpz,allimpzlev] = new_simnlimpkv(simstruct.(dgpmodel_use).simdata(:,:,isim),simstruct.(dgpmodel_use).simlev(:,:,isim),empstruct.(fitmodel_use).data,nlags,hascon,floor(ndraws/2),nstep,monte,useshock,usestpt_num,empstruct.(fitmodel_use).lev,fit_vhist,fit_exog,fit_restr,fit_impstr);
                    simstruct.(dgpmodel_use).(fitmodel_use).oilimpz(:,:,:,istpt,isim) = mean(allimpz,3);
                    simstruct.(dgpmodel_use).(fitmodel_use).cumoilimpz(:,:,:,istpt,isim) = mean(allimpzlev,3);
                end
            end
        
            simstruct.(dgpmodel_use).(fitmodel_use).cumoilimpz(:,:,:,:,isim) = cumsum(simstruct.(dgpmodel_use).(fitmodel_use).oilimpz(:,:,:,:,isim),1);
            
            if rem(isim,50)==0
               disp(' ')
               disp(['DGP ' num2str(idgp) ' of ' num2str(length(dgpmodels)) ' - Fit ' num2str(ifit) ' of ' num2str(length(fitmodels)) ' - Sim ' num2str(isim) ' of ' num2str(nsimdataset)]);
            end

        end
        
        disp(' ')
        disp(['Completed DGP ' num2str(idgp) ' of ' num2str(length(dgpmodels)) ' - Fit ' num2str(ifit) ' of ' num2str(length(fitmodels))])
    end
end

%% Section 7: Calculate empirical impulse responses
%
%Use varsim and varsim_errfeed functions to calculate impulse responses,
%given beta estimates calculated above, for the periods selected in
%infosetdates. Shock sizes are given by shkvalue(2:end), where shkvalue is
%a vector of scalar coefficients for the standard deviation of errors from
%the model. Note - the distribution errors are drawn from for non-linear
%models depends on the setting for the monte option.
%
%This section only run if calc_impulseresponse is set to 1.
%
%Fields of empstruct.[model name] (e.g. datastruct.oil_gdp) are:
%oilimpz, cumoilimpz
%
% Copied from other code - commented out the parts where other (non-oil)
% impulse responses are calculated

for imodel = 1:length(allmodels)
    
    %Select current model
    usemodel = allmodels{imodel};
    
    %Select variables for current model
    dataplus_use = empstruct.(usemodel).dataplus;
    vardatalev_use = empstruct.(usemodel).lev;
    beta_use = empstruct.(usemodel).beta;
    erz_use = empstruct.(usemodel).erz;
    vhist_use = empstruct.(usemodel).vhist;
    simopt_use = empstruct.(usemodel).simopt;
    oilix_use = empstruct.(usemodel).oilix;
    gdpix_use = empstruct.(usemodel).oilix;    
    bpix_use = empstruct.(usemodel).bpix;
    trix_use = empstruct.(usemodel).trix;
    micix_use = empstruct.(usemodel).micix;    
    
    nvars_use = size(vardatalev_use,2);
    
    signew_use = cov(empstruct.(usemodel).erz);
    facsignew_use = chol(signew_use)';
    
    for istpt = 1:nstartpoints
        
        %For each startpoints, find the stptnum to pass into varsim and
        %varsim_errfeed - i.e. the row of dataplus to use in starting to
        %choose lags.
        use_stpt = showstartpoints{istpt};
        use_stptnum = find(strcmpi(allstartpoints,use_stpt));    

        [simdatanoshock,~,errarray] = varsim(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
        
        %If models are linear, ndraws is 1, so add back in that dimension
        %to the array to get proper functionality. Otherwise, leave as-is.
        if ismember(usemodel,linmodels)
            simdata_oil = NaN([size(simdatanoshock) 1 nshocks]);            
%             simdata_gdp = NaN([size(simdatanoshock) 1 nshocks]);
%             simdata_bp = NaN([size(simdatanoshock) 1 nshocks]);
%             simdata_tr = NaN([size(simdatanoshock) 1 nshocks]);
%             simdata_mic = NaN([size(simdatanoshock) 1 nshocks]);              
        else
            simdata_oil = NaN([size(simdatanoshock) nshocks]);            
%             simdata_gdp = NaN([size(simdatanoshock) nshocks]);
%             simdata_bp = NaN([size(simdatanoshock) nshocks]);
%             simdata_tr = NaN([size(simdatanoshock) nshocks]);
%             simdata_mic = NaN([size(simdatanoshock) nshocks]);              
        end
        
        %For each shock in shkvalue(2:end), loop over the variables in the
        %model and impose this shock. Shocks are taken from the cholesky
        %decomposition of the variance-covariance matrix. Note that the
        %models all included leads so that the shocks from the cholesky
        %decomposition are identified.
        %
        %Replace the first error for all ndraw simulations with the shock
        %(replace errarray(1,ix,:) with the desired shock), and store the
        %new simulated paths.
        for ishock = 1:nshocks       
            
            if ~isempty(oilix_use)
                errarray_oilshk = errarray;
                errarray_oilshk(1,oilix_use,:) = shkvalue(ishock+1)*facsignew_use(oilix_use,oilix_use);
                simdata_oil(:,:,:,ishock) = varsim_errfeed(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,[1:nvars_use],errarray_oilshk,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
            end
            
%             if ~isempty(gdpix_use)
%                 errarray_gdpshk = errarray;
%                 errarray_gdpshk(1,gdpix_use,:) = shkvalue(ishock+1)*facsignew_use(gdpix_use,gdpix_use);
%                 simdata_gdp(:,:,:,ishock) = varsim_errfeed(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,[1:nvars_use],errarray_gdpshk,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
%             end                        
%             
%             if ~isempty(bpix_use)
%                 errarray_bpshk = errarray;
%                 errarray_bpshk(1,bpix_use,:) = shkvalue(ishock+1)*facsignew_use(bpix_use,bpix_use);
%                 simdata_bp(:,:,:,ishock) = varsim_errfeed(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,[1:nvars_use],errarray_bpshk,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
%             end
%             
%             if ~isempty(trix_use)
%                 errarray_trshk = errarray;
%                 errarray_trshk(1,trix_use,:) = shkvalue(ishock+1)*facsignew_use(trix_use,trix_use);
%                 simdata_tr(:,:,:,ishock) = varsim_errfeed(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,[1:nvars_use],errarray_trshk,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
%             end
%             
%             if ~isempty(micix_use)
%                 errarray_micshk = errarray;
%                 errarray_micshk(1,micix_use,:) = shkvalue(ishock+1)*facsignew_use(micix_use,micix_use);
%                 simdata_mic(:,:,:,ishock) = varsim_errfeed(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,[1:nvars_use],errarray_micshk,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
%             end 
        end
        
        %For each shock, and for each variable, calculate the impulse
        %response by taking the mean diffence between the no-shock and the
        %shocked simulations. Given the use of errarray and varsim_errfeed
        %above, the only difference between the two paths that are
        %subtracted is the error in the first period.
        for ishk = 1:nshocks
            if ~isempty(oilix_use)
                empstruct.(usemodel).oilimpz(:,:,ishk,istpt) = mean(simdata_oil(nlags+1:end,:,:,ishk)- simdatanoshock(nlags+1:end,:,:),3);
            else
                empstruct.(usemodel).oilimpz(:,:,ishk,istpt) = NaN(nstep,nvars_use,1,1);
            end
%             if ~isempty(gdpix_use)
%                 datastruct.(usemodel).gdpimpz(:,:,ishk,istpt) = mean(simdata_gdp(:,:,:,ishk)-simdatanoshock,3);
%             else
%                 datastruct.(usemodel).gdpimpz(:,:,ishk,istpt) = NaN(nstep,nvars_use,1,1);
%             end            
%             if ~isempty(bpix_use)
%                 datastruct.(usemodel).bpimpz(:,:,ishk,istpt) = mean(simdata_bp(:,:,:,ishk)- simdatanoshock,3);            
%             else
%                 datastruct.(usemodel).bpimpz(:,:,ishk,istpt) = NaN(nstep,nvars_use,1,1);
%             end
%             if ~isempty(trix_use)
%                 datastruct.(usemodel).trimpz(:,:,ishk,istpt) = mean(simdata_tr(:,:,:,ishk)- simdatanoshock,3);            
%             else
%                 datastruct.(usemodel).trimpz(:,:,ishk,istpt) = NaN(nstep,nvars_use,1,1);
%             end
%             if ~isempty(micix_use)
%                 datastruct.(usemodel).micimpz(:,:,ishk,istpt) = mean(simdata_mic(:,:,:,ishk)- simdatanoshock,3);            
%             else
%                 datastruct.(usemodel).micimpz(:,:,ishk,istpt) = NaN(nstep,nvars_use,1,1);
%             end 
        end
    end
    
    empstruct.(usemodel).cumoilimpz = cumsum(empstruct.(usemodel).oilimpz,1);
%     datastruct.(usemodel).cumgdpimpz = cumsum(datastruct.(usemodel).gdpimpz,1);    
%     datastruct.(usemodel).cumbpimpz = cumsum(datastruct.(usemodel).bpimpz,1);
%     datastruct.(usemodel).cumtrimpz = cumsum(datastruct.(usemodel).trimpz,1);
%     datastruct.(usemodel).cummicimpz = cumsum(datastruct.(usemodel).micimpz,1);

    disp(' ')
    disp(['Done calculating Empirical Impulse Responses for ' usemodel])
    disp(['Model ' num2str(imodel) ' of ' num2str(length(allmodels))])
    
end

disp(' ')
disp('Done calculating all Empirical Impulse Responses') 

%% Section 8: Calculate wald statistics
%
% This section of the code assumes that shocks at the beginning were
% specified as: shkvalue = [x -2 -1 1 2]
%

waldstat_dlin = NaN(nstep,nshocks,nstartpoints);
waldstat_asym = NaN(nstep,2,nstartpoints);
wald_cross = NaN(nstep,nstartpoints,nstartpoints);

if calc_waldstats == 1

%Wald stat for linearity - is the difference between the linear impulse
%response and the non-linear impulse response significant? Use variance
%calculated from models fitted on oil_gdp data generating process to
%generate the variance-covariance matrix.
for ihist = 1:nstartpoints
    for ishock = 1:nshocks
        
        simdiff_ni3_lin = squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,ishock,ihist,:)-simstruct.oil_gdp_ni3.oil_gdp.cumoilimpz(:,2,ishock,ihist,:));
        vcovmat_dlin = cov(simdiff_ni3_lin');
    
        for istep = 1:nstep
            lin_cumoilimpz = empstruct.oil_gdp.cumoilimpz(1:istep,2,ishock,ihist);
            ni3_cumoilimpz = empstruct.oil_gdp_ni3.cumoilimpz(1:istep,2,ishock,ihist);
            diff_cumoilimpz = ni3_cumoilimpz - lin_cumoilimpz;
            waldstat_dlin(istep,ishock,ihist) = diff_cumoilimpz'*inv(vcovmat_dlin(1:istep,1:istep))*diff_cumoilimpz;   
        end
    end
end

%Wald stat for asymmetry - are positive and negative shocks of equal
%magnitude but opposite sign?
for ihist = 1:nstartpoints
    
    simdiff_1std = squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,2,ihist,:)+simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,3,ihist,:));
    simdiff_2std = squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,1,ihist,:)+simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,4,ihist,:));    
    vcovmat_asym_1std = cov(simdiff_1std');
    vcovmat__asym_2std = cov(simdiff_2std');
    empdiff_1std = empstruct.oil_gdp_ni3.cumoilimpz(:,2,2,ihist)+empstruct.oil_gdp_ni3.cumoilimpz(:,2,3,ihist);
    empdiff_2std = empstruct.oil_gdp_ni3.cumoilimpz(:,2,1,ihist)+empstruct.oil_gdp_ni3.cumoilimpz(:,2,4,ihist);            
    
    for istep = 1:nstep
        waldstat_asym(istep,1,ihist) = empdiff_1std(1:istep,:)'*inv(vcovmat_asym_1std(1:istep,1:istep))*empdiff_1std(1:istep,:);
        waldstat_asym(istep,2,ihist) = empdiff_2std(1:istep,:)'*inv(vcovmat__asym_2std(1:istep,1:istep))*empdiff_2std(1:istep,:);
    end
    
end

for zx = 1:length(infosetdates)-1
    for zy = (zx+1):length(infosetdates)

        simdiff_cross_1std = squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,3,zx,:)) - squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,3,zy,:));
        simdiff_cross_2std = squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,4,zx,:)) - squeeze(simstruct.oil_gdp_ni3.oil_gdp_ni3.cumoilimpz(:,2,4,zy,:));
        var_cross_1std = cov(simdiff_cross_1std');
        var_cross_2std = cov(simdiff_cross_2std');
        empdiff_cross_1std = empstruct.oil_gdp_ni3.cumoilimpz(:,2,3,zx) - empstruct.oil_gdp_ni3.cumoilimpz(:,2,3,zy);
        empdiff_cross_2std = empstruct.oil_gdp_ni3.cumoilimpz(:,2,4,zx) - empstruct.oil_gdp_ni3.cumoilimpz(:,2,4,zy);

        for istep = 1:nstep
            wald_cross(istep,1,zx,zy) = empdiff_cross_1std(1:istep,:)'*inv(var_cross_1std(1:istep,1:istep))*empdiff_cross_1std(1:istep,:);
            wald_cross(istep,2,zx,zy) = empdiff_cross_2std(1:istep,:)'*inv(var_cross_2std(1:istep,1:istep))*empdiff_cross_2std(1:istep,:);
        end
    
    end
end

end

%% Section 9: Save data

if savedata == 1 || isdeployed
    save(outFileName,'empstruct','simstruct',...
        'infosetdates','showstartpoints','allstartpoints','allinfosetdates','effdate','leveffdate','setEstimationPeriod',...
        'allmodels','linmodels','ni3models','altmodels','fitmodels','dgpmodels',...
        'nstartpoints','ndraws','nshocks','nlags','nsimdataset','hascon',...
        'monte','nstep','useRngSeed','shkvalue','useRngSeed')
end
