%Clean script

if ~isdeployed
    clearvars
    close('all')
end

%% User Inputs

%Choose models to run, by grouping: linear, net increase 3 year, and
%alternative net increase 3 year.
%
% Constructing model name:
%  var1_var2_var3_modelType (any combination of valid variable types, plus a model type at the end)
%  where variables may be: 
%   gdp: log real US GDP
%   oil: log US imported refiner acquisition cost, deflated by US CPI
%   bp: bond premium, from Gilchrist & Zagrajsek (2012)
%    bp1 - converted to quarterly by first of period
%    bp3 - converted to quarterly by last of period
%    bpavg - converted to quarterly by period average
%   tr: treasury spread
%    tr1 - converted to quarterly by first of period
%    tr3 - converted to quarterly by last of period
%    travg - converted to quarterly by period average
%   mic: Michigan consumer confidence
% and modelType may be:
%   lin: linear
%   ni3: 3-year net increase measure, where net increase is applied to
%    first variable in system (so make sure first variable is oil). net
%    increase measure affects all variables.
%   altni3: 3-year net increase measure, where net increase is applied to
%    first variable in system (so make sure first variable is oil). net
%    increase measure affects only second variable (which is almost always GDP).
%  **Note, if no model type is given, model type is set to linear

%Group models into linmodels, ni3models, and altmodels - important for
%proper functionality

% linmodels = {'gdp_lin','oil_gdp','gdp_bp1','gdp_travg','gdp_mic','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3'};

% linmodels = {'gdp_lin','oil_gdp','gdp_bp1','gdp_travg','gdp_mic','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3'};

% linmodels = {'oil_gdp','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_bp1_ni3','oil_gdp_travg_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3'};

linmodels = {'oil_gdp','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic','oil_gdp_hsts','oil_gdp_bp1_travg_mic_hsts'};
ni3models = {'oil_gdp_ni3','oil_gdp_bp1_ni3','oil_gdp_travg_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3','oil_gdp_hsts_ni3','oil_gdp_bp1_travg_mic_hsts_ni3'};
altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3','oil_gdp_hsts_altni3','oil_gdp_bp1_travg_mic_hsts_altni3'};
 

allmodels = [linmodels ni3models altmodels ];

% Set parameters for VARs and Impulse Responses

%Number of lags to include in VAR estimation
nlags = 4;
%If hascon = 1, include constant in regression.
hascon = 1;
%For varsim, varsim_errfeed, and new_nlimpkv functions, ndraws is the number 
%of simulations to run inside function. Is the number of simulations over
%which to take mean forecast/impulse response for non-linear model, and the
%number of simulations over which to calculate variance for all model
%types.
ndraws = 200000;
%ndraws = 100;
%Number of steps, starting from 0 to nstep-1, ahead to calculate forecasts
%and impulse responses
nstep = 20;
%if monte = 0, draw from empirical distribution. If monte = 1, draw from 
%normal distribution, scaled to variance of model errors.
monte = 0;
%shkvalue - first entry has no meaning, second through last entries are
%number of standard deviations of shock to impose.
shkvalue = [1 -2 -1 1 2];
%Set the random number generator
useRngSeed = 35400;
rng(useRngSeed)

%Date range string (e.g. 1980Q1:1999Q4) for period to use in differenced
%data in VAR
%to use full sample, set to []
setEstimationPeriod = [];

%Choose infosetdates (i.e. Omega_{t-1}) to run. A given infosetdate is the
%last period of lagged values used in calculating a forecast/impulse
%response/historical decomposition/variance decomposition.
%To use all available infosetdates , set to []
%
% Note - infosetdate must be a date in the range selected by
% setEstimationPeriod, given current set up of code. It is feasible,
% however, to change the code so that betas are estimated on one set of
% dates, and impulse responses/historical decompositions/variance
% decompositions still have the full set of dates available to them.

% infosetdates = {'1979Q1','1979Q4','1980Q4','1981Q2','1990Q2','1990Q3','1994Q3',...
%     '1995Q4','1996Q3','1999Q4','2001Q1','2004Q2','2004Q4','2007Q2','2007Q4','2008Q4'};

infosetdates = [];

outFileName = 'outputOfCleanScript_allmodels_alldates';
outFileName = [outFileName '_' lower(datestr(now,'ddmmmyyyy'))];
savedata = 1;

%for speed, toggle whether to calculate impulse responses or not, and
%whether to calculate historical decompositions/variance decompositions or
%not
calc_impulseresponse = 1;
calc_histAndVarDecomp = 1;

disp('Started script')

nshocks = length(shkvalue)-1;

%% Section 1: Load in data
%
% 1. Load in data for oil, gdp, and Bond Premiums. Convert to desired
% values - for oil, calculate real oil log and log difference, converting to
% quarterly. For GDP, real gdp log and log difference, and for bond premium
% convert level to quarterly. Oil and Bond premium can be converted by
% beginning or end of period (see cnvrtbase_oil and cnvrtbase_bp)
%
% 2. Merge series by date. GDP on its own runs (in levels) from 1947Q1 to
% 2012Q2. Oil on its own runs (in levels) from Jan1974 (1974Q1) to Aug2012 
% (2012Q2). Bond Premiums run from Jan1971 (1971Q1) to Sep2010 (2010Q3).
% Also create a date vector/date string (Matlab Formats) for each of these.
%
% 3. Using the series from 1974Q1 to 2012Q2, set data, data level, data
% plus (i.e. including non-linear oil terms), beta estimates, VAR in-sample
% errors, set oilindex and crindex (which column of data corresponds to
% oil and cr, respectively), and estimation options (to be used by varsim
% and releated programs) for a variety of models: oil_gdp, gdp_bp,
% oil_gdp_bp, oil_gdp_ni3, oil_gdp_bp_ni3, and oil_gdp_bp_altni3.

%Set conversion basis for monthly to quarterly: beginning or end
cnvrtbase_oil = 'end';
cnvrtbase_bp = 'beg';

if strcmpi(cnvrtbase_oil,'beg')
    baseper_oil = 1;
elseif strcmpi(cnvrtbase_oil,'end')
    baseper_oil = 3;
end

if strcmpi(cnvrtbase_bp,'beg')
    baseper_bp = 1;
elseif strcmpi(cnvrtbase_bp,'end')
    baseper_bp = 3;
end

%if ~isdeployed
 %   restoredefaultpath
  %  addpath('T:\TQS\Vigfusson\matlabcommon')
   % addpath('T:\TQS\Vigfusson\matlabcommon\varcode\')
   % addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\')
    %addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\tvresp')
    %addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\tvresp\creditspread')
%end

oildatafile = 'mnt_racandcpi_12202013.csv';
gdpdatafile = 'qtr_rgdp_12202013.csv';

%Prepare real oil prices series
[data,varnames,rawdates] = tblread(oildatafile,'comma'); %#ok<ASGLU>
poilm_date = [rawdates(:,1:3) repmat('19',[size(rawdates,1) 1]) rawdates(:,end-1:end)];
tmpindex = find(strcmpi(cellstr(rawdates(:,end-1:end)),'00'),1,'first');
tmplen = length(poilm_date(tmpindex:end,:));
poilm_date(tmpindex:end,end-3:end-2) = repmat('20',[tmplen 1]);
poilm_date = cellstr(poilm_date);

poilm = data(:,1);
poilm_cpi = data(:,2);

%This data will start in Jan1974 and end in Aug2012
tmpindex = ~isnan(poilm);
poilm = log(poilm(tmpindex));
poilm_cpi = log(poilm_cpi(tmpindex));
poilm_date = poilm_date(tmpindex);

%Convert oil from monthly to quarterly taking period-end value
%Data will be 1974Q1 to 2012Q2
poilq = poilm(baseper_oil:3:end);
poilq_cpi = poilm_cpi(baseper_oil:3:end);
poilq_date = cellstr(datestr(datenum(poilm_date(baseper_oil:3:end),'mmmyyyy'),'yyyyqq'));

levrpoil = poilq-poilq_cpi;
levrpoil_date = poilq_date;

levnompoil = poilq;
dnompoil = diff(poilq);

drpoil = diff(poilq) - diff(poilq_cpi);
drpoil_date = poilq_date(2:end);

disp(' ')
disp('Oil data in')

%Prepare real gdp series
[data,varnames,rawdates] = tblread(gdpdatafile,'comma'); %#ok<ASGLU>

if strcmpi(gdpdatafile,'qlutzv2012m10.csv')
    levrgdp_date = [repmat('19',[length(rawdates) 1]) rawdates(:,1:2) repmat('Q',[length(rawdates) 1]) rawdates(:,end)];
    tmpindex = find(strcmpi(cellstr(rawdates(:,1:2)),'00'),1,'first');
    tmplen = length(rawdates(tmpindex:end,1));
    levrgdp_date(tmpindex:end,1:2) = repmat('20',[tmplen 1]);
    levrgdp_date = cellstr(levrgdp_date);
    
    levrgdp = log(data(:,2));
    tmpindex = ~isnan(levrgdp);
    levrgdp = levrgdp(tmpindex);
    levrgdp_date = levrgdp_date(tmpindex);
    
elseif strcmpi(gdpdatafile,'qtr_rgdp_12202013.csv')
    levrgdp_date = cellstr(rawdates);
    levrgdp = log(data(:,1));
    tmpindex = ~isnan(levrgdp);
    levrgdp = levrgdp(tmpindex);
    levrgdp_date = levrgdp_date(tmpindex);
end

disp(' ')
disp('GDP data in')

%This data will start in 1947Q1 and end in 2012Q2

drgdp = diff(levrgdp);
drgdp_date = levrgdp_date(2:end);

%Prepare Bond Premium series
[data,varnames,rawdates] = tblread('GZ_M_updated.csv','comma');
%Data starts in JAN1973 and ends in DEC2012
bondpremm_date = cellstr(datestr(datenum(rawdates,'mmm-yy'),'mmmyyyy'));

bondpremm = data(:,2);

tmpindex = ~isnan(bondpremm);
bondpremm = bondpremm(tmpindex);
bondpremm_date = bondpremm_date(tmpindex);

%This data will run from 1973Q1 to 2012Q4
bondpremq = bondpremm(baseper_bp:3:end);
bondpremq_1 = bondpremm(1:3:end);
bondpremq_3 = bondpremm(3:3:end);
bondpremq_avg = (1/3)*(bondpremm(1:3:end)+bondpremm(2:3:end)+bondpremm(3:3:end));
bondpremq_date = unique(cellstr(datestr(datenum(bondpremm_date(baseper_bp:3:end),'mmmyyyy'),'yyyyqq')));

disp(' ')
disp('Bond data in')

[data,varnames,rawdates] = tblread('treasyield.csv','comma');

treasm_date = cellstr(rawdates);
treasq_date = unique(cellstr(datestr(datenum(treasm_date,'mmmyyyy'),'yyyyqq')));

treasspreadm = data(:,1) - data(:,2); %10 year minus 5 year;
treasspreadq_1 = treasspreadm(1:3:end);
treasspreadq_3 = treasspreadm(3:3:end);
treasspreadq_avg = (1/3)*(treasspreadm(1:3:end)+treasspreadm(2:3:end)+treasspreadm(3:3:end));

disp(' ')
disp('Treas data in')

[data,varnames,qtrs] = tblread('survey.csv','comma');

michiconfidence_date = cellstr([num2str(data(2:end,1)) repmat('Q',[size(data,1)-1 1]) qtrs(2:end,1)]);
michiconfidence = data(2:end,2);
michiconfidence_log = log(michiconfidence);
michiconfidence_log_date = michiconfidence_date;
michiconfidence_dlog = diff(michiconfidence_log);
michiconfidence_dlog_date = michiconfidence_date(2:end);

disp(' ')
disp('Mich data in')


hstartsdatafile = 'houst.csv';
 
%Prepare real oil prices series
[data,varnames,hs_rawdates] = tblread(hstartsdatafile,'comma');  
hstarts = log(data(3:3:end));
hstartsq_date = unique(cellstr(datestr(datenum(hs_rawdates(3:3:end,:),'mmmyyyy'),'yyyyqq')));

disp(' ')
disp('Housing Starts in')




%Get dates where all data is available (1974Q2 to 2012Q2)
effdate = intersect(intersect(drgdp_date,drpoil_date,'stable'),bondpremq_date,'stable');
leveffdate = intersect(intersect(levrgdp_date,levrpoil_date,'stable'),bondpremq_date,'stable');

effdate = intersect(effdate,treasq_date,'stable');
leveffdate = intersect(leveffdate,treasq_date,'stable');

effdate = intersect(effdate,michiconfidence_date,'stable');
leveffdate = intersect(leveffdate,michiconfidence_date,'stable');

[~,drpoilindex] = intersect(drpoil_date,effdate,'stable');
[~,drgdpindex] = intersect(drgdp_date,effdate,'stable');
[~,bpindex] = intersect(bondpremq_date,effdate,'stable');
[~,trindex] = intersect(treasq_date,effdate,'stable');
[~,micindex] = intersect(michiconfidence_date,effdate,'stable');
[~,levrpoilindex] = intersect(levrpoil_date,leveffdate,'stable');
[~,levrgdpindex] = intersect(levrgdp_date,leveffdate,'stable');
[~,levbpindex] = intersect(bondpremq_date,leveffdate,'stable');
[~,levtrindex] = intersect(treasq_date,leveffdate,'stable');
[~,levmicindex] = intersect(michiconfidence_date,leveffdate,'stable');


[~,hstindex] = intersect(hstartsq_date ,effdate,'stable');
[~,levhstindex] = intersect(hstartsq_date ,leveffdate,'stable');

 

drpoil_reg = drpoil(drpoilindex);
levrpoil_reg = levrpoil(levrpoilindex);

dnompoil_reg = dnompoil(drpoilindex);
levnompoil_reg = levnompoil(levrpoilindex);

drgdp_reg = drgdp(drgdpindex);
levrgdp_reg = levrgdp(levrgdpindex);

bondpremq_1_reg = bondpremq_1(bpindex);
levbondpremq_1_reg = bondpremq_1(levbpindex);

bondpremq_3_reg = bondpremq_3(bpindex);
levbondpremq_3_reg = bondpremq_3(levbpindex);

bondpremq_avg_reg = bondpremq_avg(bpindex);
levbondpremq_avg_reg = bondpremq_avg(levbpindex);

treasspreadq_1_reg = treasspreadq_1(trindex);
levtreasspreadq_1_reg = treasspreadq_1(levtrindex);

treasspreadq_3_reg = treasspreadq_3(trindex);
levtreasspreadq_3_reg = treasspreadq_3(levtrindex);

treasspreadq_avg_reg = treasspreadq_avg(trindex);
levtreasspreadq_avg_reg = treasspreadq_avg(levtrindex);

michiconfidence_reg = michiconfidence(micindex);
levmichiconfidence_reg = michiconfidence(levmicindex);

hstarts_reg = hstarts(hstindex);
levhstarts_reg = hstarts(levhstindex);

 


%Start structures for holding output and parameters for each model
%Begin by identifying the order of variables (gdp, oil, bond premium) in
%the columns of data. If a variable is not included, give empty array.

allvariablematrix = [drpoil_reg, dnompoil_reg, drgdp_reg,...
                    bondpremq_1_reg, bondpremq_3_reg, bondpremq_avg_reg,...
                    treasspreadq_1_reg, treasspreadq_3_reg, treasspreadq_avg_reg,...
                    michiconfidence_reg,hstarts_reg ];
allvariablematrix_lev = [levrpoil_reg, levnompoil_reg, levrgdp_reg,...
                    levbondpremq_1_reg, levbondpremq_3_reg, levbondpremq_avg_reg,...
                    levtreasspreadq_1_reg, levtreasspreadq_3_reg, levtreasspreadq_avg_reg,...
                    levmichiconfidence_reg,levhstarts_reg];                
                
allvariablestubs = {'oil','nomoil','gdp','bp1','bp3','bpavg','tr1','tr3','travg','mic','hsts'};
oilvariablestubs = {'oil','nomoil'};
bpvariablestubs = {'bp1','bp3','bpavg'};
trvariablestubs = {'tr1','tr3','travg'};
allmodelstubs = {'lin','ni3','altni3'};

%% Section 2: Organize data for models to run
%Prepare data and level for each of the selected models, by selecting
%appropriate columns of allvariablematrix and allvariablematrix_lev. Then
%assign variable column index value (e.g. gdpix, oilix) fields for the
%each model. If a variable is not included in the model, then its ix field
%is set to []
%
%Fields assigned for datastruct.[model name]
%data, lev, gdpix, oilix, bpix, trix, micix

for imodel = 1:length(allmodels)
    
    usemodel = allmodels{imodel};
    
    score_index = strfind(usemodel,'_');
    storevars = cell(length(score_index)+1,1);
    
    for ivar = 1:length(score_index)+1;
        if ivar == 1
            storevars{ivar} = usemodel(1:score_index(1)-1);
        elseif ivar == length(score_index)+1
            storevars{ivar} = usemodel(score_index(ivar-1)+1:end);
        else
            storevars{ivar} = usemodel(score_index(ivar-1)+1:score_index(ivar)-1);
        end
    end
    
    if any(~ismember(storevars,union(allvariablestubs,allmodelstubs)))
        error('Invald input string for model')
    end
    
    storevars = storevars(~ismember(storevars,allmodelstubs));    
    
    if any(strcmpi(storevars,'gdp'))
       if ~isscalar(find(strcmpi(storevars,'gdp'),1))
           error('Invalid input string for model - more than one gdp input.')
       else        
            datastruct.(usemodel).gdpix = find(strcmpi(storevars,'gdp'),1);
       end
    else
        datastruct.(usemodel).gdpix = [];
    end
    
    if any(ismember(storevars,oilvariablestubs))
       if ~isscalar(find(ismember(storevars,oilvariablestubs),1))
           error('Invalid input string for model - more than one oil input.')
       else        
            datastruct.(usemodel).oilix = find(ismember(storevars,oilvariablestubs),1);
       end
    else
        datastruct.(usemodel).oilix = [];
    end
    
    if any(ismember(storevars,bpvariablestubs))
       if ~isscalar(find(ismember(storevars,bpvariablestubs)))
           error('Invalid input string for model - more than one bond premium variable.')
       else
           datastruct.(usemodel).bpix = find(ismember(storevars,bpvariablestubs));
       end
    else
        datastruct.(usemodel).bpix = [];
    end
    
    if any(ismember(storevars,trvariablestubs))
       if ~isscalar(find(ismember(storevars,trvariablestubs)))
           error('Invalid input string for model - more than one bond premium variable.')
       else
           datastruct.(usemodel).trix = find(ismember(storevars,trvariablestubs));
       end
    else
        datastruct.(usemodel).trix = [];
    end
    
    if any(strcmpi(storevars,'mic'))
        if ~isscalar(find(strcmpi(storevars,'mic'),1))
            error('Invalid input string for model - more than one gdp input.')
        else
            datastruct.(usemodel).micix = find(strcmpi(storevars,'mic'),1);
        end
    else
        datastruct.(usemodel).micix = [];
    end
    
    
    
    if any(strcmpi(storevars,'hsts'))
        if ~isscalar(find(strcmpi(storevars,'hsts'),1))
            error('Invalid input string for model - more than one gdp input.')
        else
            datastruct.(usemodel).hstsix = find(strcmpi(storevars,'hsts'),1);
        end
    else
        datastruct.(usemodel).hstsix = [];
    end

    
    mapdatamatrix = zeros(length(allvariablestubs),length(storevars));
    
    for ivar = 1:length(storevars);
        mapdatamatrix(strcmpi(storevars(ivar),allvariablestubs),ivar) = 1;
    end
    
    datastruct.(usemodel).data = allvariablematrix*mapdatamatrix;
    datastruct.(usemodel).lev = allvariablematrix_lev*mapdatamatrix;
    
end

%% Section 3: select estimation period, and check infosetdates are valide
%
%Given input for setEstimationPeriod, find the indecies of effdate and
%leveffdate (all dates where we have all the data we loaded in)
%corresponding to setEstimationPeriod. Then check that infosetdates are in
%this sample period, conditional on number of lags for models.

if isempty(setEstimationPeriod);
    useEstimationPeriod_index = 1:length(effdate);
    useEstimationPeriodLev_index = 1:length(leveffdate);
else
    colon_index = strfind(setEstimationPeriod,':');
    startEstPeriod = setEstimationPeriod(1:colon_index-1);
    endEstPeriod = setEstimationPeriod(colon_index+1:end);
    startEstPeriod_index = find(strcmpi(startEstPeriod,effdate)==1);
    endEstPeriod_index = find(strcmpi(endEstPeriod,effdate)==1);    
    useEstimationPeriod_index = startEstPeriod_index:endEstPeriod_index;
    startEstPeriodLev_index = find(strcmpi(startEstPeriod,leveffdate)==1)-1;
    endEstPeriodLev_index = find(strcmpi(endEstPeriod,leveffdate)==1);
    useEstimationPeriodLev_index = startEstPeriodLev_index:endEstPeriodLev_index;    
end

if isempty(useEstimationPeriod_index)
    error('Input dates for setEstimationPeriod not found in effdate.')
end

if isempty(useEstimationPeriodLev_index)
    error('Input dates for setEstimationPeriod not found in effdate.')
end

%Use nlags and effdate to determine number and dates of all possible
%startpoints. Convention for naming startpoints is the first forecast
%period based on that start point. infosetdates (i.e. Omega_{t-1}) is the
%last period of lags 

allstartpoints = effdate(nlags+1:end); %Date of first forecast period of all startpoints

%For historical decomposition, we find regression errors depending on what
%we used to calculate betas - so need to find the subset of startpoints
%that we will have errors for.
effdate_estsubset = effdate(useEstimationPeriod_index);
allstartpoints_estsubset = effdate_estsubset(nlags+1:end);

%Technically, last start point would forecast 1st period outside the 
%sample. Add this to allstartpoints.
[tmp_yr,tmp_mnt] = datevec(allstartpoints{end},'yyyyqq');
allstartpoints = [allstartpoints; cellstr(datestr(datenum(tmp_yr,tmp_mnt+3,1),'yyyyqq'))];

[tmp_yr,tmp_mnt] = datevec(allstartpoints_estsubset{end},'yyyyqq');
allstartpoints_estsubset = [allstartpoints_estsubset; cellstr(datestr(datenum(tmp_yr,tmp_mnt+3,1),'yyyyqq'))];

%Infosetdates are one quarter prior to start points
allinfosetdates = cell(length(allstartpoints),1);
for idate = 1:length(allstartpoints)
    allinfosetdates(idate) = cellstr(datestr(addtodate(datenum(allstartpoints(idate),'yyyyqq'),-3,'month'),'yyyyqq'));
end

allinfosetdates_estsubset = cell(length(allstartpoints_estsubset),1);
for idate = 1:length(allstartpoints_estsubset)
    allinfosetdates_estsubset(idate) = cellstr(datestr(addtodate(datenum(allstartpoints_estsubset(idate),'yyyyqq'),-3,'month'),'yyyyqq'));
end

%If infosetdates is empty, use all the legal infosetdates
if isempty(infosetdates)
    infosetdates = allinfosetdates_estsubset;
else
    
    %otherwise, check that input infosetdates are legal given estimation
    %period
    if any(~ismember(infosetdates,allinfosetdates_estsubset))
        error('One or more infosetdates supplied by user is not available given the input data and setting for setEstimationPeriod')
    end
end

%Generate list of start points based on list of infosetdates provided above
showstartpoints = cell(1,length(infosetdates));
for idate = 1:length(infosetdates)
    showstartpoints{idate} = datestr(addtodate(datenum(infosetdates{idate},'yyyyqq'),3,'month'),'yyyyqq');
end

nstartpoints = length(showstartpoints);

%% Section 4: Estimate beta for model, given estimation period
%
%For linear, 3-year net increase, and alternate 3-year net increase models,
%use modelinput function to find options for vhist, betaopt, and simopt.
%Given these option settings, use new_estimatemodel function to calculate
%dataplus (i.e. data with non-linear term appended as additional column),
%beta estimate, and regression errors.
%
%Fields for datastruct.[model name] (e.g. datastruct.oil_gdp) filled in:
%vhist, betaopt, simopt, beta, erz, dataplus
%

%Run VAR for linear models, and store results/options in structure
[vhist_lin,betaopt_lin,simopt_lin] = modelinput('lin_cu'); %modelinput: lin_cu --> linear model, with leads for cholesky decomposition

for imodel = 1:length(linmodels)
    
    datastruct.(linmodels{imodel}).vhist = vhist_lin;
    datastruct.(linmodels{imodel}).betaopt = betaopt_lin;    
    datastruct.(linmodels{imodel}).simopt = simopt_lin;
    
    [beta_lin,erz_bp,dataplus_lin] = new_estimatemodel(datastruct.(linmodels{imodel}).data(useEstimationPeriod_index,:),datastruct.(linmodels{imodel}).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_lin,betaopt_lin);
    
    datastruct.(linmodels{imodel}).beta = beta_lin;
    datastruct.(linmodels{imodel}).erz = erz_bp;
    datastruct.(linmodels{imodel}).dataplus = dataplus_lin;    

end

%Run VAR for Net Increase model, and store results/options in structure
[vhist_ni3,betaopt_ni3,simopt_ni3] = modelinput('ham3_cu'); %ham3_cu --> 3-year net increase model, with leads for cholesky decomposition

for imodel = 1:length(ni3models)
    
    datastruct.(ni3models{imodel}).vhist = vhist_ni3;
    datastruct.(ni3models{imodel}).betaopt = betaopt_ni3;    
    datastruct.(ni3models{imodel}).simopt = simopt_ni3;
    
    [beta_ni3,erz_ni3,dataplus_ni3] = new_estimatemodel(datastruct.(ni3models{imodel}).data(useEstimationPeriod_index,:),datastruct.(ni3models{imodel}).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_ni3,betaopt_ni3);
    
    datastruct.(ni3models{imodel}).beta = beta_ni3;
    datastruct.(ni3models{imodel}).erz = erz_ni3;
    datastruct.(ni3models{imodel}).dataplus = dataplus_ni3;    

end

%Construct beta and erz for Alternative Net Increase model by using results
%from oil_gdp_bp_ni3 and oil_gdp_bp calculated above

for imodel = 1:length(altmodels)
    
    usemodel_alt = altmodels{imodel};
    score_index = strfind(usemodel_alt,'_');
    usemodel_lin = usemodel_alt(1:score_index(end)-1);
    usemodel_ni3 = [usemodel_lin '_ni3'];
    
    datastruct.(usemodel_alt).vhist = vhist_ni3;
    datastruct.(usemodel_alt).betaopt = betaopt_ni3;
    datastruct.(usemodel_alt).simopt = simopt_ni3;
    
    if ismember(usemodel_lin,fields(datastruct))
        usebeta_lin = datastruct.(usemodel_lin).beta;
        useerz_lin = datastruct.(usemodel_lin).erz;
    else
        [usebeta_lin,useerz_lin] = new_estimatemodel(datastruct.(usemodel_alt).data(useEstimationPeriod_index,:),datastruct.(usemodel_alt).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_lin,betaopt_lin);
    end
    
    if ismember(usemodel_ni3,fields(datastruct))
        usebeta_ni3 = datastruct.(usemodel_ni3).beta;
        useerz_ni3 = datastruct.(usemodel_ni3).erz;
        usedataplus_ni3 = datastruct.(usemodel_ni3).dataplus;        
    else
        [usebeta_ni3,useerz_ni3,usedataplus_ni3] = new_estimatemodel(datastruct.(usemodel_alt).data(useEstimationPeriod_index,:),datastruct.(usemodel_alt).lev(useEstimationPeriodLev_index,:),nlags,hascon,[],vhist_ni3,betaopt_ni3);        
    end

    %Alternative errors are same as linear for Bond Premium, and net increase
    %for others
    erz_alt_tmp = useerz_ni3;
    erz_alt_tmp(:,3:end) = useerz_lin(:,3:end);    

    %Coefficient estimates are same as linear for Bond Premium, and net increase
    %for others. Linear coefficients must be rearranged, and zeros placed for
    %net increase terms.
    
    nvars = size(usebeta_ni3,1);
    nvars_exoil_exgdp = nvars-2;
    
    beta_alt_tmp = NaN(size(usebeta_ni3));
    beta_alt_tmp([1 2],:) = usebeta_ni3([1 2],:); %All coefficients in oil and gdp rows
    beta_alt_tmp(3:end,1:1+nvars*nlags) = usebeta_lin(3:end,1:1+nvars*nlags); %Coefficients for constant and lags of dependents
    beta_alt_tmp(3:end,1+nvars*nlags+(1:nlags)) = zeros(nvars_exoil_exgdp,nlags); %Net increase lags get zero coefficient
    beta_alt_tmp(3:end,1+(nvars+1)*nlags+1) = usebeta_lin(3:end,1+nvars*nlags+1); %Lead oil term
    beta_alt_tmp(3:end,1+(nvars+1)*nlags+2) = zeros(nvars_exoil_exgdp,1); %Lead for net increase gets zero coefficient
    beta_alt_tmp(3:end,1+(nvars+1)*nlags+(3:nvars)) = usebeta_lin(3:end,1+nvars*nlags+(2:nvars-1)); %Lead gdp and other variable terms

    datastruct.(usemodel_alt).beta = beta_alt_tmp;
    datastruct.(usemodel_alt).erz = erz_alt_tmp;
    datastruct.(usemodel_alt).dataplus = usedataplus_ni3;
    
end

%% Section 5: Calculate Impulse responses
%
%Use varsim and varsim_errfeed functions to calculate impulse responses,
%given beta estimates calculated above, for the periods selected in
%infosetdates. Shock sizes are given by shkvalue(2:end), where shkvalue is
%a vector of scalar coefficients for the standard deviation of errors from
%the model. Note - the distribution errors are drawn from for non-linear
%models depends on the setting for the monte option.
%
%This section only run if calc_impulseresponse is set to 1.
%
%Fields of datastruct.[model name] (e.g. datastruct.oil_gdp) are:
%oilimpz, cumoilimpz, gdpimpz, cumgdpimpz, bpimpz, cumbpimpz, trimpz,
%cumtrimpz, micimpz, cummicimpz
%

if calc_impulseresponse == 1

for imodel = 1:length(allmodels)
    
    %Select current model
    usemodel = allmodels{imodel};
    
    %Select variables for current model
    dataplus_use = datastruct.(usemodel).dataplus;
    vardatalev_use = datastruct.(usemodel).lev;
    beta_use = datastruct.(usemodel).beta;
    erz_use = datastruct.(usemodel).erz;
    vhist_use = datastruct.(usemodel).vhist;
    simopt_use = datastruct.(usemodel).simopt;
    oilix_use = datastruct.(usemodel).oilix;
    gdpix_use = datastruct.(usemodel).oilix;    
    bpix_use = datastruct.(usemodel).bpix;
    trix_use = datastruct.(usemodel).trix;
    micix_use = datastruct.(usemodel).micix;    
    hstsix_use = datastruct.(usemodel).hstsix;    

    nvars_use = size(vardatalev_use,2);
    
    signew_use = cov(datastruct.(usemodel).erz);
    facsignew_use = chol(signew_use)';
    tempoilimpz = NaN(nstep,nvars_use, nshocks, nstartpoints);
    for istpt = 1:nstartpoints
        
        %For each startpoints, find the stptnum to pass into varsim and
        %varsim_errfeed - i.e. the row of dataplus to use in starting to
        %choose lags.
        use_stpt = showstartpoints{istpt};
        use_stptnum = find(strcmpi(allstartpoints,use_stpt));    

        [simdatanoshock,~,errarray] = varsim(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
        
        %If models are linear, ndraws is 1, so add back in that dimension
        %to the array to get proper functionality. Otherwise, leave as-is.
        if ismember(usemodel,linmodels)
            simdata_oil = NaN([size(simdatanoshock) 1 nshocks]);            
%             simdata_gdp = NaN([size(simdatanoshock) 1 nshocks]);
%             simdata_bp = NaN([size(simdatanoshock) 1 nshocks]);
%             simdata_tr = NaN([size(simdatanoshock) 1 nshocks]);
%             simdata_mic = NaN([size(simdatanoshock) 1 nshocks]);              
        else
            simdata_oil = NaN([size(simdatanoshock) 1]);            
%             simdata_gdp = NaN([size(simdatanoshock) nshocks]);
%             simdata_bp = NaN([size(simdatanoshock) nshocks]);
%             simdata_tr = NaN([size(simdatanoshock) nshocks]);
%             simdata_mic = NaN([size(simdatanoshock) nshocks]);              
        end
        
        %For each shock in shkvalue(2:end), loop over the variables in the
        %model and impose this shock. Shocks are taken from the cholesky
        %decomposition of the variance-covariance matrix. Note that the
        %models all included leads so that the shocks from the cholesky
        %decomposition are identified.
        %
        %Replace the first error for all ndraw simulations with the shock
        %(replace errarray(1,ix,:) with the desired shock), and store the
        %new simulated paths.
        facsigtemp = facsignew_use(oilix_use,oilix_use);
        simdatanoshocktemp = simdatanoshock(nlags+1:end,:,:);
        tempAPS = zeros(nstep,nvars_use,nshocks);
        warning('off','MATLAB:mir_warning_maybe_uninitialized_temporary');
        parfor ishock = 1:nshocks
            
            if ~isempty(oilix_use)
                errarray_oilshk = errarray;
                errarray_oilshk(1,oilix_use,:) = shkvalue(ishock+1)*facsigtemp;
                simdata_oil_tmp = varsim_errfeed(dataplus_use,vardatalev_use,beta_use,erz_use,nlags,hascon,[1:nvars_use],errarray_oilshk,ndraws,nstep,monte,use_stptnum,vhist_use,simopt_use);
            end
            tempAPS(:,:,ishock) = mean(simdata_oil_tmp(nlags+1:end,:,:)- simdatanoshocktemp,3);
            %size(tempAPS)
            %size(tempoilimpz)
        end
        tempoilimpz(:,:,:,istpt) = tempAPS;
        warning('on','MATLAB:mir_warning_maybe_uninitialized_temporary');
        
    end
    datastruct.(usemodel).oilimpz= tempoilimpz;
    datastruct.(usemodel).cumoilimpz = cumsum(datastruct.(usemodel).oilimpz,1);

    disp(' ')
    disp(['Done calculating Impulse Responses for ' usemodel])
    disp(['Model ' num2str(imodel) ' of ' num2str(length(allmodels))])
    
end

disp(' ')
disp('Done calculating all Impulse Responses') 

end

%% Section 6: Calculate Historial and Variance Decompositions and Mean Squared Errors
%
%Use varsim and varsim_errfeed functions to calculate historical and
%variance decompositions for all variables included in all models for the
%selected infosetdates.
%
%Historical decompositions:
%Historial decompositions are calculated by estimating a mean forecast for 
%a given Omega_{t-1}, i.e. set of lagged values in the data, with all random 
%errors for all variables (with errors drawn for a distribution
%depending on setting of the monte option). The mean is calculated over
%ndraws forecasts for non-linear models, and with 0 shocks for linear
%models.
%
%Then, for each variable, a new forecast is generated where instead of 
%random errors (or no shocks, for the linear model) for that variable, the 
%actual regression errors are used instead. For example, gdpfrcst is 
%generated by taking random errors for all non-gdp variables, and using 
%actual regression errors for gdp.
%
%Finally, gdperror is calculated as the difference between the unconditional
%(all random errors) forecast and the forecast conditional on knowing 
%actual shocks for gdp (gdpfrcst).
%
%In addition, totalerror is calculated as the difference between the
%unconditional mean forecast and the actual data.
%
%Variance decompositions:
%
%Again we start with using varsim to estimate ndraws forecasts with all
%random errors. In this case, for linear models, we actually use random
%errors in order to generate a distribution. We than calculate the variance
%of the forecast at each step for these ndraw simulations.
%
%Then, for each variable, we calculate the notvar forecast. In this
%instance, we keep random errors for the variable of interest while using
%actual regression errors for all other variables in the model. For
%example, the notgdpfrcst is calculated by imposing the actual regression
%errors for all non-gdp variables, and using the random errors for GDP we
%used in generating the unconditional distribution of forecasts. We then
%calculate notgdpfrcstvar by taking the variance of notgdpfrcst.
%
%Finally, the GDP contribution to forecast variance is given by
%notgdpfrcstvar./frcstvar
%
%Mean Squared Error:
%At the end of the loop over allmodels, we also calculate the 
%mean-squared total error and mean-squared value for the difference between
%the unconditional forecast and forecast conditional on GDP shocks (i.e.
%gdpfrcst).

if calc_histAndVarDecomp == 1

for imodel = 1:length(allmodels)
    
    usemodel = allmodels{imodel};
    
    nvars = size(datastruct.(usemodel).data,2);
    
    dataplus_use = datastruct.(usemodel).dataplus;
    vardatalevel_use = datastruct.(usemodel).lev;
    beta_use = datastruct.(usemodel).beta;
    erz_use = datastruct.(usemodel).erz;
    vhist_use = datastruct.(usemodel).vhist;
    simopt_use = datastruct.(usemodel).simopt;
    gdpix_use = datastruct.(usemodel).gdpix;
    oilix_use = datastruct.(usemodel).oilix;
    bpix_use = datastruct.(usemodel).bpix;
    trix_use = datastruct.(usemodel).trix;
    micix_use = datastruct.(usemodel).micix;    
    hstsix_use = datastruct.(usemodel).hstsix;    

    if simopt_use(4) == -1
        simopt_lin = simopt_use;
        simopt_use(4) = -2;
        linflag = true(1,1);
    else
        linflag = false(1,1);
    end
    
    feederror_rand = NaN(nstep,nvars,ndraws);
    
    for istep = 1:nstep;
        rr1 = rand(ndraws,1);
        tmp1 = reshape(erz_use(ceil(size(erz_use,1)*rr1),:)',[1 nvars ndraws]);
        feederror_rand(istep,:,:) = tmp1;
    end
    
    if simopt_use(4)== -1 || simopt_use(4) == -2
        feederror_rand = zeros(size(feederror_rand));
    end
    
    frcst = NaN(nstep,nvars,nstartpoints);
    cumfrcst = NaN(nstep,nvars,nstartpoints);
    perfrcst = NaN(nstep,nvars,nstartpoints);
    cumperfrcst = NaN(nstep,nvars,nstartpoints);
    gdpfrcst = NaN(nstep,nvars,nstartpoints);
    cumgdpfrcst = NaN(nstep,nvars,nstartpoints);
    notgdpfrcst = NaN(nstep,nvars,nstartpoints);
    cumnotgdpfrcst = NaN(nstep,nvars,nstartpoints);
    
    frcstvar = NaN(nstep,nvars,nstartpoints);
    cumfrcstvar = NaN(nstep,nvars,nstartpoints);
    perfrcstvar = NaN(nstep,nvars,nstartpoints);
    cumperfrcstvar = NaN(nstep,nvars,nstartpoints);    
    gdpfrcstvar = NaN(nstep,nvars,nstartpoints);
    cumgdpfrcstvar = NaN(nstep,nvars,nstartpoints);
    notgdpfrcstvar = NaN(nstep,nvars,nstartpoints);
    cumnotgdpfrcstvar = NaN(nstep,nvars,nstartpoints);    
        
    if ~isempty(oilix_use)
        oilfrcst = NaN(nstep,nvars,nstartpoints);
        cumoilfrcst = NaN(nstep,nvars,nstartpoints);
        oilfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumoilfrcstvar = NaN(nstep,nvars,nstartpoints);
        
        notoilfrcst = NaN(nstep,nvars,nstartpoints);
        cumnotoilfrcst = NaN(nstep,nvars,nstartpoints);
        notoilfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumnotoilfrcstvar = NaN(nstep,nvars,nstartpoints);        
    end
        
    if ~isempty(bpix_use)
        bpfrcst = NaN(nstep,nvars,nstartpoints);
        cumbpfrcst = NaN(nstep,nvars,nstartpoints);
        bpfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumbpfrcstvar = NaN(nstep,nvars,nstartpoints);
        
        notbpfrcst = NaN(nstep,nvars,nstartpoints);
        cumnotbpfrcst = NaN(nstep,nvars,nstartpoints);
        notbpfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumnotbpfrcstvar = NaN(nstep,nvars,nstartpoints);        
    end
    
    if ~isempty(trix_use)
        trfrcst = NaN(nstep,nvars,nstartpoints);
        cumtrfrcst = NaN(nstep,nvars,nstartpoints);
        trfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumtrfrcstvar = NaN(nstep,nvars,nstartpoints);
        
        nottrfrcst = NaN(nstep,nvars,nstartpoints);
        cumnottrfrcst = NaN(nstep,nvars,nstartpoints);
        nottrfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumnottrfrcstvar = NaN(nstep,nvars,nstartpoints);        
    end
    
    if ~isempty(micix_use)
        micfrcst = NaN(nstep,nvars,nstartpoints);
        cummicfrcst = NaN(nstep,nvars,nstartpoints);
        micfrcstvar = NaN(nstep,nvars,nstartpoints);
        cummicfrcstvar = NaN(nstep,nvars,nstartpoints);
        
        notmicfrcst = NaN(nstep,nvars,nstartpoints);
        cumnotmicfrcst = NaN(nstep,nvars,nstartpoints);
        notmicfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumnotmicfrcstvar = NaN(nstep,nvars,nstartpoints);        
    end
    
    
    
        if ~isempty(hstsix_use)
        hstsfrcst = NaN(nstep,nvars,nstartpoints);
        cumhstsfrcst = NaN(nstep,nvars,nstartpoints);
        hstsfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumhstsfrcstvar = NaN(nstep,nvars,nstartpoints);
        
        nothstsfrcst = NaN(nstep,nvars,nstartpoints);
        cumnothstsfrcst = NaN(nstep,nvars,nstartpoints);
        nothstsfrcstvar = NaN(nstep,nvars,nstartpoints);
        cumnothstsfrcstvar = NaN(nstep,nvars,nstartpoints);        
    end
    
    actualdata = NaN(nstep,nvars,nstartpoints);
    
    for istpt = 1:nstartpoints
        
        use_stpt = showstartpoints{istpt};
        use_stptnum = find(strcmpi(allstartpoints,use_stpt));
        
        test_stptlocation = find(strcmpi(allstartpoints,use_stpt));
        step_use = min(nstep,(length(allstartpoints)-test_stptlocation));        
        
        actualerz_stpt = erz_use(use_stptnum+(0:step_use-1),:);
        
        actualdata(1:step_use,:,istpt) = dataplus_use(nlags+use_stptnum+(0:step_use-1),1:nvars);

        [frcst_sim,~,feederror_sim] = varsim(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);        
        frcst(1:step_use,:,istpt) = mean(frcst_sim(nlags+1:end,:,:),3);
        frcstvar(1:step_use,:,istpt) = var(frcst_sim(nlags+1:end,:,:),[],3);        
        cumfrcst_sim = cumsum(frcst_sim(nlags+1:end,:,:),1);
        cumfrcst(1:step_use,:,istpt) = mean(cumfrcst_sim,3);
        cumfrcstvar(1:step_use,:,istpt) = var(cumfrcst_sim,[],3);
        
        if linflag
            tmpval = varsim(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
            frcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
            cumfrcst(1:step_use,:,istpt) = cumsum(frcst(1:step_use,:,istpt),1);
        end
    
        [perfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,actualerz_stpt,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
        perfrcst(1:step_use,:,istpt) = mean(perfrcst_sim(nlags+1:end,:,:),3);
        perfrcstvar(1:step_use,:,istpt) = var(perfrcst_sim(nlags+1:end,:,:),[],3);        
        cumperfrcst_sim = cumsum(perfrcst_sim(nlags+1:end,:,:),1);
        cumperfrcst(1:step_use,:,istpt) = mean(cumperfrcst_sim,3);
        cumperfrcstvar(1:step_use,:,istpt) = var(cumperfrcst_sim,[],3);
        
        if linflag
            tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,actualerz_stpt,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
            perfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
            cumperfrcst(1:step_use,:,istpt) = cumsum(perfrcst(1:step_use,:,istpt),1);
        end      
        
        gdp_feederror = feederror_sim(1:step_use,:,:);
        gdp_feederror(:,gdpix_use,:) = repmat(actualerz_stpt(:,gdpix_use),[1 1 ndraws]);
        
        [gdpfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,gdp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
        gdpfrcst(1:step_use,:,istpt) = mean(gdpfrcst_sim(nlags+1:end,:,:),3);
        gdpfrcstvar(1:step_use,:,istpt) = var(gdpfrcst_sim(nlags+1:end,:,:),[],3);        
        cumgdpfrcst_sim = cumsum(gdpfrcst_sim(nlags+1:end,:,:),1);
        cumgdpfrcst(1:step_use,:,istpt) = mean(cumgdpfrcst_sim,3);
        cumgdpfrcstvar(1:step_use,:,istpt) = var(cumgdpfrcst_sim,[],3);
        
        if linflag
            gdp_feederror = zeros(step_use,nvars);
            gdp_feederror(:,gdpix_use,:) = actualerz_stpt(:,gdpix_use);            
            
            tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,gdp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
            gdpfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
            cumgdpfrcst(1:step_use,:,istpt) = cumsum(gdpfrcst(1:step_use,:,istpt),1);
        end       
        
        notgdpix_use = setdiff(1:nvars,gdpix_use);
        notgdp_feederror = feederror_sim(1:step_use,:,:);
        notgdp_feederror(:,notgdpix_use,:) = repmat(actualerz_stpt(:,notgdpix_use),[1 1 ndraws]);
        
        [notgdpfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notgdp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
        notgdpfrcst(1:step_use,:,istpt) = mean(notgdpfrcst_sim(nlags+1:end,:,:),3);
        notgdpfrcstvar(1:step_use,:,istpt) = var(notgdpfrcst_sim(nlags+1:end,:,:),[],3);        
        cumnotgdpfrcst_sim = cumsum(notgdpfrcst_sim(nlags+1:end,:,:),1);
        cumnotgdpfrcst(1:step_use,:,istpt) = mean(cumnotgdpfrcst_sim,3);
        cumnotgdpfrcstvar(1:step_use,:,istpt) = var(cumnotgdpfrcst_sim,[],3);
        
        if linflag
            notgdp_feederror = zeros(step_use,nvars);
            notgdp_feederror(:,notgdpix_use,:) = actualerz_stpt(:,notgdpix_use);            
            
            tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notgdp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
            notgdpfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
            cumnotgdpfrcst(1:step_use,:,istpt) = cumsum(notgdpfrcst(1:step_use,:,istpt),1);
        end        
        
        if ~isempty(oilix_use)
            
            oil_feederror = feederror_sim(1:step_use,:,:);
            oil_feederror(:,oilix_use,:) = repmat(actualerz_stpt(:,oilix_use),[1 1 ndraws]);    
            
            [oilfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,oil_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            oilfrcst(1:step_use,:,istpt) = mean(oilfrcst_sim(nlags+1:end,:,:),3);
            oilfrcstvar(1:step_use,:,istpt) = var(oilfrcst_sim(nlags+1:end,:,:),[],3);        
            cumoilfrcst_sim = cumsum(oilfrcst_sim(nlags+1:end,:,:),1);
            cumoilfrcst(1:step_use,:,istpt) = mean(cumoilfrcst_sim,3);
            cumoilfrcstvar(1:step_use,:,istpt) = var(cumoilfrcst_sim,[],3);
            
            if linflag
                oil_feederror = zeros(step_use,nvars);
                oil_feederror(:,oilix_use,:) = actualerz_stpt(:,oilix_use);            
            
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,oil_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                oilfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumoilfrcst(1:step_use,:,istpt) = cumsum(oilfrcst(1:step_use,:,istpt),1);
            end            
            
            notoilix_use = setdiff(1:nvars,oilix_use);            
            notoil_feederror = feederror_sim(1:step_use,:,:);
            notoil_feederror(:,notoilix_use,:) = repmat(actualerz_stpt(:,notoilix_use),[1 1 ndraws]);    
            
            [notoilfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notoil_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            notoilfrcst(1:step_use,:,istpt) = mean(notoilfrcst_sim(nlags+1:end,:,:),3);
            notoilfrcstvar(1:step_use,:,istpt) = var(notoilfrcst_sim(nlags+1:end,:,:),[],3);        
            cumnotoilfrcst_sim = cumsum(notoilfrcst_sim(nlags+1:end,:,:),1);
            cumnotoilfrcst(1:step_use,:,istpt) = mean(cumnotoilfrcst_sim,3);
            cumnotoilfrcstvar(1:step_use,:,istpt) = var(cumnotoilfrcst_sim,[],3);
            
            if linflag
                notoil_feederror = zeros(step_use,nvars);
                notoil_feederror(:,notoilix_use,:) = actualerz_stpt(:,notoilix_use);            
            
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notoil_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                notoilfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumnotoilfrcst(1:step_use,:,istpt) = cumsum(notoilfrcst(1:step_use,:,istpt),1);
            end            
            
        end
        
        if ~isempty(bpix_use)
            
            bp_feederror = feederror_sim(1:step_use,:,:);
            bp_feederror(:,bpix_use,:) = repmat(actualerz_stpt(:,bpix_use),[1 1 ndraws]);                
            
            [bpfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,bp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            bpfrcst(1:step_use,:,istpt) = mean(bpfrcst_sim(nlags+1:end,:,:),3);
            bpfrcstvar(1:step_use,:,istpt) = var(bpfrcst_sim(nlags+1:end,:,:),[],3);        
            cumbpfrcst_sim = cumsum(bpfrcst_sim(nlags+1:end,:,:),1);
            cumbpfrcst(1:step_use,:,istpt) = mean(cumbpfrcst_sim,3);
            cumbpfrcstvar(1:step_use,:,istpt) = var(cumbpfrcst_sim,[],3);
            
            if linflag
                bp_feederror = zeros(step_use,nvars);
                bp_feederror(:,bpix_use,:) = actualerz_stpt(:,bpix_use);            
            
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,bp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                bpfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumbpfrcst(1:step_use,:,istpt) = cumsum(bpfrcst(1:step_use,:,istpt),1);
            end             
            
            notbpix_use = setdiff(1:nvars,bpix_use);            
            notbp_feederror = feederror_sim(1:step_use,:,:);
            notbp_feederror(:,notbpix_use,:) = repmat(actualerz_stpt(:,notbpix_use),[1 1 ndraws]);    
            
            [notbpfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notbp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            notbpfrcst(1:step_use,:,istpt) = mean(notbpfrcst_sim(nlags+1:end,:,:),3);
            notbpfrcstvar(1:step_use,:,istpt) = var(notbpfrcst_sim(nlags+1:end,:,:),[],3);        
            cumnotbpfrcst_sim = cumsum(notbpfrcst_sim(nlags+1:end,:,:),1);
            cumnotbpfrcst(1:step_use,:,istpt) = mean(cumnotbpfrcst_sim,3);
            cumnotbpfrcstvar(1:step_use,:,istpt) = var(cumnotbpfrcst_sim,[],3);
            
            if linflag
                notbp_feederror = zeros(step_use,nvars);
                notbp_feederror(:,notbpix_use,:) = actualerz_stpt(:,notbpix_use);            
            
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notbp_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                notbpfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumnotbpfrcst(1:step_use,:,istpt) = cumsum(notbpfrcst(1:step_use,:,istpt),1);
            end            
            
        end
        
        if ~isempty(trix_use)
            
            tr_feederror = feederror_sim(1:step_use,:,:);
            tr_feederror(:,trix_use,:) = repmat(actualerz_stpt(:,trix_use),[1 1 ndraws]);                
            
            [trfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,tr_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            trfrcst(1:step_use,:,istpt) = mean(trfrcst_sim(nlags+1:end,:,:),3);
            trfrcstvar(1:step_use,:,istpt) = var(trfrcst_sim(nlags+1:end,:,:),[],3);        
            cumtrfrcst_sim = cumsum(trfrcst_sim(nlags+1:end,:,:),1);
            cumtrfrcst(1:step_use,:,istpt) = mean(cumtrfrcst_sim,3);
            cumtrfrcstvar(1:step_use,:,istpt) = var(cumtrfrcst_sim,[],3);
            
            if linflag
                tr_feederror = zeros(step_use,nvars);
                tr_feederror(:,trix_use,:) = actualerz_stpt(:,trix_use);            
            
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,tr_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                trfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumtrfrcst(1:step_use,:,istpt) = cumsum(trfrcst(1:step_use,:,istpt),1);
            end            
            
            nottrix_use = setdiff(1:nvars,trix_use);            
            nottr_feederror = feederror_sim(1:step_use,:,:);
            nottr_feederror(:,nottrix_use,:) = repmat(actualerz_stpt(:,nottrix_use),[1 1 ndraws]);    
            
            [nottrfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,nottr_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            nottrfrcst(1:step_use,:,istpt) = mean(nottrfrcst_sim(nlags+1:end,:,:),3);
            nottrfrcstvar(1:step_use,:,istpt) = var(nottrfrcst_sim(nlags+1:end,:,:),[],3);        
            cumnottrfrcst_sim = cumsum(nottrfrcst_sim(nlags+1:end,:,:),1);
            cumnottrfrcst(1:step_use,:,istpt) = mean(cumnottrfrcst_sim,3);
            cumnottrfrcstvar(1:step_use,:,istpt) = var(cumnottrfrcst_sim,[],3);
            
            if linflag
                nottr_feederror = zeros(step_use,nvars);
                nottr_feederror(:,nottrix_use,:) = actualerz_stpt(:,nottrix_use);            
            
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,nottr_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                nottrfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumnottrfrcst(1:step_use,:,istpt) = cumsum(nottrfrcst(1:step_use,:,istpt),1);
            end 
            
        end
        
        if ~isempty(micix_use)
            
            mic_feederror = feederror_sim(1:step_use,:,:);
            mic_feederror(:,micix_use,:) = repmat(actualerz_stpt(:,micix_use),[1 1 ndraws]);
            
            [micfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,mic_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            micfrcst(1:step_use,:,istpt) = mean(micfrcst_sim(nlags+1:end,:,:),3);
            micfrcstvar(1:step_use,:,istpt) = var(micfrcst_sim(nlags+1:end,:,:),[],3);
            cummicfrcst_sim = cumsum(micfrcst_sim(nlags+1:end,:,:),1);
            cummicfrcst(1:step_use,:,istpt) = mean(cummicfrcst_sim,3);
            cummicfrcstvar(1:step_use,:,istpt) = var(cummicfrcst_sim,[],3);
            
            if linflag
                mic_feederror = zeros(step_use,nvars);
                mic_feederror(:,micix_use,:) = actualerz_stpt(:,micix_use);
                
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,mic_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                micfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cummicfrcst(1:step_use,:,istpt) = cumsum(micfrcst(1:step_use,:,istpt),1);
            end
            
            notmicix_use = setdiff(1:nvars,micix_use);
            notmic_feederror = feederror_sim(1:step_use,:,:);
            notmic_feederror(:,notmicix_use,:) = repmat(actualerz_stpt(:,notmicix_use),[1 1 ndraws]);
            
            [notmicfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notmic_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            notmicfrcst(1:step_use,:,istpt) = mean(notmicfrcst_sim(nlags+1:end,:,:),3);
            notmicfrcstvar(1:step_use,:,istpt) = var(notmicfrcst_sim(nlags+1:end,:,:),[],3);
            cumnotmicfrcst_sim = cumsum(notmicfrcst_sim(nlags+1:end,:,:),1);
            cumnotmicfrcst(1:step_use,:,istpt) = mean(cumnotmicfrcst_sim,3);
            cumnotmicfrcstvar(1:step_use,:,istpt) = var(cumnotmicfrcst_sim,[],3);
            
            if linflag
                notmic_feederror = zeros(step_use,nvars);
                notmic_feederror(:,notmicix_use,:) = actualerz_stpt(:,notmicix_use);
                
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,notmic_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                notmicfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumnotmicfrcst(1:step_use,:,istpt) = cumsum(notmicfrcst(1:step_use,:,istpt),1);
            end
            
        end
        
        
        
        
        
        if ~isempty(hstsix_use)
            
            hsts_feederror = feederror_sim(1:step_use,:,:);
            hsts_feederror(:,hstsix_use,:) = repmat(actualerz_stpt(:,hstsix_use),[1 1 ndraws]);
            
            [hstsfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,hsts_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            hstsfrcst(1:step_use,:,istpt) = mean(hstsfrcst_sim(nlags+1:end,:,:),3);
            hstsfrcstvar(1:step_use,:,istpt) = var(hstsfrcst_sim(nlags+1:end,:,:),[],3);
            cumhstsfrcst_sim = cumsum(hstsfrcst_sim(nlags+1:end,:,:),1);
            cumhstsfrcst(1:step_use,:,istpt) = mean(cumhstsfrcst_sim,3);
            cumhstsfrcstvar(1:step_use,:,istpt) = var(cumhstsfrcst_sim,[],3);
            
            if linflag
                hsts_feederror = zeros(step_use,nvars);
                hsts_feederror(:,hstsix_use,:) = actualerz_stpt(:,hstsix_use);
                
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,hsts_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                hstsfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumhstsfrcst(1:step_use,:,istpt) = cumsum(hstsfrcst(1:step_use,:,istpt),1);
            end
            
            nothstsix_use = setdiff(1:nvars,hstsix_use);
            nothsts_feederror = feederror_sim(1:step_use,:,:);
            nothsts_feederror(:,nothstsix_use,:) = repmat(actualerz_stpt(:,nothstsix_use),[1 1 ndraws]);
            
            [nothstsfrcst_sim,~] = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,nothsts_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_use);
            nothstsfrcst(1:step_use,:,istpt) = mean(nothstsfrcst_sim(nlags+1:end,:,:),3);
            nothstsfrcstvar(1:step_use,:,istpt) = var(nothstsfrcst_sim(nlags+1:end,:,:),[],3);
            cumnothstsfrcst_sim = cumsum(nothstsfrcst_sim(nlags+1:end,:,:),1);
            cumnothstsfrcst(1:step_use,:,istpt) = mean(cumnothstsfrcst_sim,3);
            cumnothstsfrcstvar(1:step_use,:,istpt) = var(cumnothstsfrcst_sim,[],3);
            
            if linflag
                nothsts_feederror = zeros(step_use,nvars);
                nothsts_feederror(:,nothstsix_use,:) = actualerz_stpt(:,nothstsix_use);
                
                tmpval = varsim_errfeed(dataplus_use,vardatalevel_use,beta_use,erz_use,nlags,hascon,1:nvars,nothsts_feederror,ndraws,step_use,monte,use_stptnum,vhist_use,simopt_lin);
                nothstsfrcst(1:step_use,:,istpt) = tmpval(nlags+1:end,:);
                cumnothstsfrcst(1:step_use,:,istpt) = cumsum(nothstsfrcst(1:step_use,:,istpt),1);
            end
            
        end

            
    end
    
    cumactualdata = cumsum(actualdata,1);
    
    datastruct.(usemodel).actualdata = actualdata;
    datastruct.(usemodel).cumactualdata = cumactualdata;    
    
    datastruct.(usemodel).frcst = frcst;
    datastruct.(usemodel).cumfrcst = cumfrcst;
    datastruct.(usemodel).frcstvar = frcstvar;
    datastruct.(usemodel).cumfrcstvar = cumfrcstvar;    
    datastruct.(usemodel).perfrcst = perfrcst;
    datastruct.(usemodel).cumperfrcst = cumperfrcst;
    datastruct.(usemodel).perfrcstvar = perfrcstvar;
    datastruct.(usemodel).cumperfrcstvar = cumperfrcstvar;    
    datastruct.(usemodel).gdpfrcst = gdpfrcst;
    datastruct.(usemodel).cumgdpfrcst = cumgdpfrcst;
    datastruct.(usemodel).notgdpfrcst = notgdpfrcst;
    datastruct.(usemodel).cumnotgdpfrcst = cumnotgdpfrcst;    
    datastruct.(usemodel).gdpfrcstvar = gdpfrcstvar;
    datastruct.(usemodel).cumgdpfrcstvar = cumgdpfrcstvar;
    datastruct.(usemodel).notgdpfrcst = notgdpfrcst;
    datastruct.(usemodel).cumnotgdpfrcst = cumnotgdpfrcst;
    datastruct.(usemodel).notgdpfrcstvar = notgdpfrcstvar;
    datastruct.(usemodel).cumnotgdpfrcstvar = cumnotgdpfrcstvar;       
    
    datastruct.(usemodel).totalerror = actualdata - frcst;    
    datastruct.(usemodel).cumtotalerror = cumactualdata - cumfrcst;    
    datastruct.(usemodel).perferror = perfrcst - frcst;    
    datastruct.(usemodel).cumperferror = cumperfrcst - cumfrcst;
    datastruct.(usemodel).gdperror = gdpfrcst - frcst;
    datastruct.(usemodel).cumgdperror = cumgdpfrcst - cumfrcst;
    datastruct.(usemodel).gdpvarcntrb = notgdpfrcstvar./frcstvar;
    datastruct.(usemodel).cumgdpvarcntrb = cumnotgdpfrcstvar./cumfrcstvar;    
    
    if ~isempty(oilix_use)
        datastruct.(usemodel).oilfrcst = oilfrcst;
        datastruct.(usemodel).cumoilfrcst = cumoilfrcst;
        datastruct.(usemodel).notoilfrcst = notoilfrcst;
        datastruct.(usemodel).cumnotoilfrcst = cumnotoilfrcst;        
        datastruct.(usemodel).oilfrcstvar = oilfrcstvar;
        datastruct.(usemodel).cumoilfrcstvar = cumoilfrcstvar;
        datastruct.(usemodel).notoilfrcstvar = notoilfrcstvar;
        datastruct.(usemodel).cumnotoilfrcstvar = cumnotoilfrcstvar;        
        
        datastruct.(usemodel).oilerror = oilfrcst - frcst;
        datastruct.(usemodel).cumoilerror = cumoilfrcst - cumfrcst;
        datastruct.(usemodel).oilvarcntrb = notoilfrcstvar./frcstvar;
        datastruct.(usemodel).cumoilvarcntrb = cumnotoilfrcstvar./cumfrcstvar;       
        
    else
        datastruct.(usemodel).oilfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumoilfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).notoilfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnotoilfrcst = NaN(nstep,nvars,nstartpoints);        
        datastruct.(usemodel).oilfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumoilfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).notoilfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnotoilfrcstvar = NaN(nstep,nvars,nstartpoints);        
        
        datastruct.(usemodel).oilerror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumoilerror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).oilvarcntrb = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumoilvarcntrb = NaN(nstep,nvars,nstartpoints);        
    end
    
    if ~isempty(bpix_use)
        datastruct.(usemodel).bpfrcst = bpfrcst;
        datastruct.(usemodel).cumbpfrcst = cumbpfrcst;
        datastruct.(usemodel).notbpfrcst = notbpfrcst;
        datastruct.(usemodel).cumnotbpfrcst = cumnotbpfrcst;        
        datastruct.(usemodel).bpfrcstvar = bpfrcstvar;
        datastruct.(usemodel).cumbpfrcstvar = cumbpfrcstvar;
        datastruct.(usemodel).notbpfrcstvar = notbpfrcstvar;
        datastruct.(usemodel).cumnotbpfrcstvar = cumnotbpfrcstvar;        
        
        datastruct.(usemodel).bperror = bpfrcst - frcst;
        datastruct.(usemodel).cumbperror = cumbpfrcst - cumfrcst;
        datastruct.(usemodel).bpvarcntrb = notbpfrcstvar./frcstvar;
        datastruct.(usemodel).cumbpvarcntrb = cumnotbpfrcstvar./cumfrcstvar;       
        
    else
        datastruct.(usemodel).bpfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumbpfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).notbpfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnotbpfrcst = NaN(nstep,nvars,nstartpoints);        
        datastruct.(usemodel).bpfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumbpfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).notbpfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnotbpfrcstvar = NaN(nstep,nvars,nstartpoints);        
        
        datastruct.(usemodel).bperror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumbperror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).bpvarcntrb = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumbpvarcntrb = NaN(nstep,nvars,nstartpoints);        
    end
    
    if ~isempty(trix_use)
        datastruct.(usemodel).trfrcst = trfrcst;
        datastruct.(usemodel).cumtrfrcst = cumtrfrcst;
        datastruct.(usemodel).nottrfrcst = nottrfrcst;
        datastruct.(usemodel).cumnottrfrcst = cumnottrfrcst;        
        datastruct.(usemodel).trfrcstvar = trfrcstvar;
        datastruct.(usemodel).cumtrfrcstvar = cumtrfrcstvar;
        datastruct.(usemodel).nottrfrcstvar = nottrfrcstvar;
        datastruct.(usemodel).cumnottrfrcstvar = cumnottrfrcstvar;        
        
        datastruct.(usemodel).trerror = trfrcst - frcst;
        datastruct.(usemodel).cumtrerror = cumtrfrcst - cumfrcst;
        datastruct.(usemodel).trvarcntrb = nottrfrcstvar./frcstvar;
        datastruct.(usemodel).cumtrvarcntrb = cumnottrfrcstvar./cumfrcstvar;       
        
    else
        datastruct.(usemodel).trfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumtrfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).nottrfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnottrfrcst = NaN(nstep,nvars,nstartpoints);        
        datastruct.(usemodel).trfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumtrfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).nottrfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnottrfrcstvar = NaN(nstep,nvars,nstartpoints);        
        
        datastruct.(usemodel).trerror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumtrerror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).trvarcntrb = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumtrvarcntrb = NaN(nstep,nvars,nstartpoints);        
    end   
    
    if ~isempty(micix_use)
        datastruct.(usemodel).micfrcst = micfrcst;
        datastruct.(usemodel).cummicfrcst = cummicfrcst;
        datastruct.(usemodel).notmicfrcst = notmicfrcst;
        datastruct.(usemodel).cumnotmicfrcst = cumnotmicfrcst;        
        datastruct.(usemodel).micfrcstvar = micfrcstvar;
        datastruct.(usemodel).cummicfrcstvar = cummicfrcstvar;
        datastruct.(usemodel).notmicfrcstvar = notmicfrcstvar;
        datastruct.(usemodel).cumnotmicfrcstvar = cumnotmicfrcstvar;        
        
        datastruct.(usemodel).micerror = micfrcst - frcst;
        datastruct.(usemodel).cummicerror = cummicfrcst - cumfrcst;
        datastruct.(usemodel).micvarcntrb = notmicfrcstvar./frcstvar;
        datastruct.(usemodel).cummicvarcntrb = cumnotmicfrcstvar./cumfrcstvar;       
        
    else
        datastruct.(usemodel).micfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cummicfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).notmicfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnotmicfrcst = NaN(nstep,nvars,nstartpoints);        
        datastruct.(usemodel).micfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cummicfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).notmicfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnotmicfrcstvar = NaN(nstep,nvars,nstartpoints);        
        
        datastruct.(usemodel).micerror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cummicerror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).micvarcntrb = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cummicvarcntrb = NaN(nstep,nvars,nstartpoints);        
    end    
    
    
    
    
    
    
    
    
    
        
    if ~isempty(hstsix_use)
        datastruct.(usemodel).hstsfrcst = hstsfrcst;
        datastruct.(usemodel).cumhstsfrcst = cumhstsfrcst;
        datastruct.(usemodel).nothstsfrcst = nothstsfrcst;
        datastruct.(usemodel).cumnothstsfrcst = cumnothstsfrcst;        
        datastruct.(usemodel).hstsfrcstvar = hstsfrcstvar;
        datastruct.(usemodel).cumhstsfrcstvar = cumhstsfrcstvar;
        datastruct.(usemodel).nothstsfrcstvar = nothstsfrcstvar;
        datastruct.(usemodel).cumnothstsfrcstvar = cumnothstsfrcstvar;        
        
        datastruct.(usemodel).hstserror = hstsfrcst - frcst;
        datastruct.(usemodel).cumhstserror = cumhstsfrcst - cumfrcst;
        datastruct.(usemodel).hstsvarcntrb = nothstsfrcstvar./frcstvar;
        datastruct.(usemodel).cumhstsvarcntrb = cumnothstsfrcstvar./cumfrcstvar;       
        
    else
        datastruct.(usemodel).hstsfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumhstsfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).nothstsfrcst = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnothstsfrcst = NaN(nstep,nvars,nstartpoints);        
        datastruct.(usemodel).hstsfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumhstsfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).nothstsfrcstvar = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumnothstsfrcstvar = NaN(nstep,nvars,nstartpoints);        
        
        datastruct.(usemodel).hstserror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumhstserror = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).hstsvarcntrb = NaN(nstep,nvars,nstartpoints);
        datastruct.(usemodel).cumhstsvarcntrb = NaN(nstep,nvars,nstartpoints);        
    end 
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    datastruct.(usemodel).mserror = NaN(nstep,nvars,nstartpoints);
    datastruct.(usemodel).cummserror = NaN(nstep,nvars,nstartpoints); 
    
    for istpt = 1:nstartpoints
        nstep_use = sum(~isnan((datastruct.(usemodel).totalerror(:,1,istpt))),1);
        
        datastruct.(usemodel).mserror(1:nstep_use,:,istpt) = cumsum((100*datastruct.(usemodel).totalerror(1:nstep_use,:,istpt)).^2,1)./repmat(transpose(1:nstep_use),[1 nvars]);
        datastruct.(usemodel).cummserror(1:nstep_use,:,istpt) = cumsum((100*datastruct.(usemodel).cumtotalerror(1:nstep_use,:,istpt)).^2,1)./repmat(transpose(1:nstep_use),[1 nvars]);
        
        datastruct.(usemodel).mserror_gdperror(1:nstep_use,:,istpt) = cumsum((100*datastruct.(usemodel).gdperror(1:nstep_use,:,istpt)).^2,1)./repmat(transpose(1:nstep_use),[1 nvars]);
        datastruct.(usemodel).cummserror_gdperror(1:nstep_use,:,istpt) = cumsum((100*datastruct.(usemodel).cumgdperror(1:nstep_use,:,istpt)).^2,1)./repmat(transpose(1:nstep_use),[1 nvars]);
        
        datastruct.(usemodel).mserror_9(:,istpt) = sum((100*datastruct.(usemodel).totalerror(1:9,:,istpt)).^2,1)/9;
        datastruct.(usemodel).cummserror_9(:,istpt) = sum((100*datastruct.(usemodel).cumtotalerror(1:9,:,istpt)).^2,1)/9; 
        
        datastruct.(usemodel).mserror_9_gdp(:,istpt) = sum((100*datastruct.(usemodel).gdperror(1:9,:,istpt)).^2,1)/9;
        datastruct.(usemodel).cummserror_9_gdp(:,istpt) = sum((100*datastruct.(usemodel).cumgdperror(1:9,:,istpt)).^2,1)/9;        
    end
    
    disp(' ')
    disp(['Done calculating Historical Decompositions and Variance Decompostions for ' usemodel])
    disp(['Model ' num2str(imodel) ' of ' num2str(length(allmodels))])
    
end

disp(' ')
disp('Done calculating all Historical Decompositions and Variance Decompositions')

end

%% Section 7: Save data

if savedata == 1 || isdeployed
    save(outFileName,'datastruct',...
        'infosetdates','showstartpoints','allstartpoints','allinfosetdates','effdate','leveffdate','setEstimationPeriod',...
        'allmodels','linmodels','ni3models','altmodels',...
        'nstartpoints','ndraws','nshocks','nlags','hascon',...
        'monte','nstep','useRngSeed','shkvalue','useRngSeed')
end



