%Clean script

if ~isdeployed
    clearvars
    close('all')
end

%% User Inputs

%Choose models to run, by grouping: linear, net increase 3 year, and
%alternative net increase 3 year.
%
% Constructing model name:
%  var1_var2_var3_modelType (any combination of valid variable types, plus a model type at the end)
%  where variables may be: 
%   gdp: log real US GDP
%   oil: log US imported refiner acquisition cost, deflated by US CPI
%   bp: bond premium, from Gilchrist & Zagrajsek (2012)
%    bp1 - converted to quarterly by first of period
%    bp3 - converted to quarterly by last of period
%    bpavg - converted to quarterly by period average
%   tr: treasury spread
%    tr1 - converted to quarterly by first of period
%    tr3 - converted to quarterly by last of period
%    travg - converted to quarterly by period average
%   mic: Michigan consumer confidence
% and modelType may be:
%   lin: linear
%   ni3: 3-year net increase measure, where net increase is applied to
%    first variable in system (so make sure first variable is oil). net
%    increase measure affects all variables.
%   altni3: 3-year net increase measure, where net increase is applied to
%    first variable in system (so make sure first variable is oil). net
%    increase measure affects only second variable (which is almost always GDP).
%  **Note, if no model type is given, model type is set to linear

%Group models into linmodels, ni3models, and altmodels - important for
%proper functionality

% linmodels = {'gdp_lin','oil_gdp','gdp_bp1','gdp_travg','gdp_mic','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3'};

% linmodels = {'gdp_lin','oil_gdp','gdp_bp1','gdp_travg','gdp_mic','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3'};

% linmodels = {'oil_gdp','oil_gdp_bp1','oil_gdp_travg','oil_gdp_mic','oil_gdp_bp1_travg_mic'};
% ni3models = {'oil_gdp_ni3','oil_gdp_bp1_ni3','oil_gdp_travg_ni3','oil_gdp_mic_ni3','oil_gdp_bp1_travg_mic_ni3'};
% altmodels = {'oil_gdp_bp1_altni3','oil_gdp_travg_altni3','oil_gdp_mic_altni3','oil_gdp_bp1_travg_mic_altni3'};

linmodels = {'oil_gdp_dhsts','oil_gdp_bp1_travg_mic_dhsts'};
ni3models = {'oil_gdp_dhsts_ni3','oil_gdp_bp1_travg_mic_dhsts_ni3'};
altmodels = {'oil_gdp_dhsts_altni3','oil_gdp_bp1_travg_mic_dhsts_altni3'};
 

allmodels = [linmodels ni3models altmodels ];

% Set parameters for VARs and Impulse Responses

%Number of lags to include in VAR estimation
nlags = 4;
%If hascon = 1, include constant in regression.
hascon = 1;
%For varsim, varsim_errfeed, and new_nlimpkv functions, ndraws is the number 
%of simulations to run inside function. Is the number of simulations over
%which to take mean forecast/impulse response for non-linear model, and the
%number of simulations over which to calculate variance for all model
%types.
ndraws = 200000;
%ndraws = 100;
%Number of steps, starting from 0 to nstep-1, ahead to calculate forecasts
%and impulse responses
nstep = 20;
%if monte = 0, draw from empirical distribution. If monte = 1, draw from 
%normal distribution, scaled to variance of model errors.
monte = 0;
%shkvalue - first entry has no meaning, second through last entries are
%number of standard deviations of shock to impose.
shkvalue = [1 -2 -1 1 2];
%Set the random number generator
useRngSeed = 35400;
rng(useRngSeed)

%Date range string (e.g. 1980Q1:1999Q4) for period to use in differenced
%data in VAR
%to use full sample, set to []
setEstimationPeriod = [];

%Choose infosetdates (i.e. Omega_{t-1}) to run. A given infosetdate is the
%last period of lagged values used in calculating a forecast/impulse
%response/historical decomposition/variance decomposition.
%To use all available infosetdates , set to []
%
% Note - infosetdate must be a date in the range selected by
% setEstimationPeriod, given current set up of code. It is feasible,
% however, to change the code so that betas are estimated on one set of
% dates, and impulse responses/historical decompositions/variance
% decompositions still have the full set of dates available to them.

% infosetdates = {'1979Q1','1979Q4','1980Q4','1981Q2','1990Q2','1990Q3','1994Q3',...
%     '1995Q4','1996Q3','1999Q4','2001Q1','2004Q2','2004Q4','2007Q2','2007Q4','2008Q4'};

infosetdates = [];

outFileName = 'outputOfCleanScript_dh_alldates';
outFileName = [outFileName '_' lower(datestr(now,'ddmmmyyyy'))];
savedata = 1;

%for speed, toggle whether to calculate impulse responses or not, and
%whether to calculate historical decompositions/variance decompositions or
%not
calc_impulseresponse = 1;
calc_histAndVarDecomp = 1;

disp('Started script')

nshocks = length(shkvalue)-1;

%% Section 1: Load in data
%
% 1. Load in data for oil, gdp, and Bond Premiums. Convert to desired
% values - for oil, calculate real oil log and log difference, converting to
% quarterly. For GDP, real gdp log and log difference, and for bond premium
% convert level to quarterly. Oil and Bond premium can be converted by
% beginning or end of period (see cnvrtbase_oil and cnvrtbase_bp)
%
% 2. Merge series by date. GDP on its own runs (in levels) from 1947Q1 to
% 2012Q2. Oil on its own runs (in levels) from Jan1974 (1974Q1) to Aug2012 
% (2012Q2). Bond Premiums run from Jan1971 (1971Q1) to Sep2010 (2010Q3).
% Also create a date vector/date string (Matlab Formats) for each of these.
%
% 3. Using the series from 1974Q1 to 2012Q2, set data, data level, data
% plus (i.e. including non-linear oil terms), beta estimates, VAR in-sample
% errors, set oilindex and crindex (which column of data corresponds to
% oil and cr, respectively), and estimation options (to be used by varsim
% and releated programs) for a variety of models: oil_gdp, gdp_bp,
% oil_gdp_bp, oil_gdp_ni3, oil_gdp_bp_ni3, and oil_gdp_bp_altni3.

%Set conversion basis for monthly to quarterly: beginning or end
cnvrtbase_oil = 'end';
cnvrtbase_bp = 'beg';

if strcmpi(cnvrtbase_oil,'beg')
    baseper_oil = 1;
elseif strcmpi(cnvrtbase_oil,'end')
    baseper_oil = 3;
end

if strcmpi(cnvrtbase_bp,'beg')
    baseper_bp = 1;
elseif strcmpi(cnvrtbase_bp,'end')
    baseper_bp = 3;
end

%if ~isdeployed
 %   restoredefaultpath
  %  addpath('T:\TQS\Vigfusson\matlabcommon')
   % addpath('T:\TQS\Vigfusson\matlabcommon\varcode\')
   % addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\')
    %addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\tvresp')
    %addpath('T:\TQS\Vigfusson\commodity\lutz\frcst2012\tvresp\creditspread')
%end

oildatafile = 'mnt_racandcpi_12202013.csv';
gdpdatafile = 'qtr_rgdp_12202013.csv';

%Prepare real oil prices series
[data,varnames,rawdates] = tblread(oildatafile,'comma'); %#ok<ASGLU>
poilm_date = [rawdates(:,1:3) repmat('19',[size(rawdates,1) 1]) rawdates(:,end-1:end)];
tmpindex = find(strcmpi(cellstr(rawdates(:,end-1:end)),'00'),1,'first');
tmplen = length(poilm_date(tmpindex:end,:));
poilm_date(tmpindex:end,end-3:end-2) = repmat('20',[tmplen 1]);
poilm_date = cellstr(poilm_date);

poilm = data(:,1);
poilm_cpi = data(:,2);

%This data will start in Jan1974 and end in Aug2012
tmpindex = ~isnan(poilm);
poilm = log(poilm(tmpindex));
poilm_cpi = log(poilm_cpi(tmpindex));
poilm_date = poilm_date(tmpindex);

%Convert oil from monthly to quarterly taking period-end value
%Data will be 1974Q1 to 2012Q2
poilq = poilm(baseper_oil:3:end);
poilq_cpi = poilm_cpi(baseper_oil:3:end);
poilq_date = cellstr(datestr(datenum(poilm_date(baseper_oil:3:end),'mmmyyyy'),'yyyyqq'));

levrpoil = poilq-poilq_cpi;
levrpoil_date = poilq_date;

levnompoil = poilq;
dnompoil = diff(poilq);

drpoil = diff(poilq) - diff(poilq_cpi);
drpoil_date = poilq_date(2:end);

disp(' ')
disp('Oil data in')

%Prepare real gdp series
[data,varnames,rawdates] = tblread(gdpdatafile,'comma'); %#ok<ASGLU>

if strcmpi(gdpdatafile,'qlutzv2012m10.csv')
    levrgdp_date = [repmat('19',[length(rawdates) 1]) rawdates(:,1:2) repmat('Q',[length(rawdates) 1]) rawdates(:,end)];
    tmpindex = find(strcmpi(cellstr(rawdates(:,1:2)),'00'),1,'first');
    tmplen = length(rawdates(tmpindex:end,1));
    levrgdp_date(tmpindex:end,1:2) = repmat('20',[tmplen 1]);
    levrgdp_date = cellstr(levrgdp_date);
    
    levrgdp = log(data(:,2));
    tmpindex = ~isnan(levrgdp);
    levrgdp = levrgdp(tmpindex);
    levrgdp_date = levrgdp_date(tmpindex);
    
elseif strcmpi(gdpdatafile,'qtr_rgdp_12202013.csv')
    levrgdp_date = cellstr(rawdates);
    levrgdp = log(data(:,1));
    tmpindex = ~isnan(levrgdp);
    levrgdp = levrgdp(tmpindex);
    levrgdp_date = levrgdp_date(tmpindex);
end

disp(' ')
disp('GDP data in')

%This data will start in 1947Q1 and end in 2012Q2

drgdp = diff(levrgdp);
drgdp_date = levrgdp_date(2:end);

%Prepare Bond Premium series
[data,varnames,rawdates] = tblread('GZ_M_updated.csv','comma');
%Data starts in JAN1973 and ends in DEC2012
bondpremm_date = cellstr(datestr(datenum(rawdates,'mmm-yy'),'mmmyyyy'));

bondpremm = data(:,2);

tmpindex = ~isnan(bondpremm);
bondpremm = bondpremm(tmpindex);
bondpremm_date = bondpremm_date(tmpindex);

%This data will run from 1973Q1 to 2012Q4
bondpremq = bondpremm(baseper_bp:3:end);
bondpremq_1 = bondpremm(1:3:end);
bondpremq_3 = bondpremm(3:3:end);
bondpremq_avg = (1/3)*(bondpremm(1:3:end)+bondpremm(2:3:end)+bondpremm(3:3:end));
bondpremq_date = unique(cellstr(datestr(datenum(bondpremm_date(baseper_bp:3:end),'mmmyyyy'),'yyyyqq')));

disp(' ')
disp('Bond data in')

[data,varnames,rawdates] = tblread('treasyield.csv','comma');

treasm_date = cellstr(rawdates);
treasq_date = unique(cellstr(datestr(datenum(treasm_date,'mmmyyyy'),'yyyyqq')));

treasspreadm = data(:,1) - data(:,2); %10 year minus 5 year;
treasspreadq_1 = treasspreadm(1:3:end);
treasspreadq_3 = treasspreadm(3:3:end);
treasspreadq_avg = (1/3)*(treasspreadm(1:3:end)+treasspreadm(2:3:end)+treasspreadm(3:3:end));

disp(' ')
disp('Treas data in')

[data,varnames,qtrs] = tblread('survey.csv','comma');

michiconfidence_date = cellstr([num2str(data(2:end,1)) repmat('Q',[size(data,1)-1 1]) qtrs(2:end,1)]);
michiconfidence = data(2:end,2);
michiconfidence_log = log(michiconfidence);
michiconfidence_log_date = michiconfidence_date;
michiconfidence_dlog = diff(michiconfidence_log);
michiconfidence_dlog_date = michiconfidence_date(2:end);

disp(' ')
disp('Mich data in')


hstartsdatafile = 'houst.csv';
 
%Prepare real oil prices series
[data,varnames,hs_rawdates] = tblread(hstartsdatafile,'comma');  
hstarts = log(data(3:3:end));
hstartsq_date = unique(cellstr(datestr(datenum(hs_rawdates(3:3:end,:),'mmmyyyy'),'yyyyqq')));
hstarts_dlog= diff(hstarts);
hstartsq_dlog_date = hstartsq_date(2:end); 



disp(' ')
disp('Housing Starts in')




%Get dates where all data is available (1974Q2 to 2012Q2)
effdate = intersect(intersect(drgdp_date,drpoil_date,'stable'),bondpremq_date,'stable');
leveffdate = intersect(intersect(levrgdp_date,levrpoil_date,'stable'),bondpremq_date,'stable');

effdate = intersect(effdate,treasq_date,'stable');
leveffdate = intersect(leveffdate,treasq_date,'stable');

effdate = intersect(effdate,michiconfidence_date,'stable');
leveffdate = intersect(leveffdate,michiconfidence_date,'stable');

[~,drpoilindex] = intersect(drpoil_date,effdate,'stable');
[~,drgdpindex] = intersect(drgdp_date,effdate,'stable');
[~,bpindex] = intersect(bondpremq_date,effdate,'stable');
[~,trindex] = intersect(treasq_date,effdate,'stable');
[~,micindex] = intersect(michiconfidence_date,effdate,'stable');
[~,levrpoilindex] = intersect(levrpoil_date,leveffdate,'stable');
[~,levrgdpindex] = intersect(levrgdp_date,leveffdate,'stable');
[~,levbpindex] = intersect(bondpremq_date,leveffdate,'stable');
[~,levtrindex] = intersect(treasq_date,leveffdate,'stable');
[~,levmicindex] = intersect(michiconfidence_date,leveffdate,'stable');


[~,hstindex] = intersect(hstartsq_dlog_date ,effdate,'stable');
[~,levhstindex] = intersect(hstartsq_date ,leveffdate,'stable');

 

drpoil_reg = drpoil(drpoilindex);
levrpoil_reg = levrpoil(levrpoilindex);

dnompoil_reg = dnompoil(drpoilindex);
levnompoil_reg = levnompoil(levrpoilindex);

drgdp_reg = drgdp(drgdpindex);
levrgdp_reg = levrgdp(levrgdpindex);

bondpremq_1_reg = bondpremq_1(bpindex);
levbondpremq_1_reg = bondpremq_1(levbpindex);

bondpremq_3_reg = bondpremq_3(bpindex);
levbondpremq_3_reg = bondpremq_3(levbpindex);

bondpremq_avg_reg = bondpremq_avg(bpindex);
levbondpremq_avg_reg = bondpremq_avg(levbpindex);

treasspreadq_1_reg = treasspreadq_1(trindex);
levtreasspreadq_1_reg = treasspreadq_1(levtrindex);

treasspreadq_3_reg = treasspreadq_3(trindex);
levtreasspreadq_3_reg = treasspreadq_3(levtrindex);

treasspreadq_avg_reg = treasspreadq_avg(trindex);
levtreasspreadq_avg_reg = treasspreadq_avg(levtrindex);

michiconfidence_reg = michiconfidence(micindex);
levmichiconfidence_reg = michiconfidence(levmicindex);

hstarts_reg = hstarts_dlog(hstindex);
levhstarts_reg = hstarts(levhstindex);

 


%Start structures for holding output and parameters for each model
%Begin by identifying the order of variables (gdp, oil, bond premium) in
%the columns of data. If a variable is not included, give empty array.

allvariablematrix = [drpoil_reg, dnompoil_reg, drgdp_reg,...
                    bondpremq_1_reg, bondpremq_3_reg, bondpremq_avg_reg,...
                    treasspreadq_1_reg, treasspreadq_3_reg, treasspreadq_avg_reg,...
                    michiconfidence_reg,hstarts_reg ];
allvariablematrix_lev = [levrpoil_reg, levnompoil_reg, levrgdp_reg,...
                    levbondpremq_1_reg, levbondpremq_3_reg, levbondpremq_avg_reg,...
                    levtreasspreadq_1_reg, levtreasspreadq_3_reg, levtreasspreadq_avg_reg,...
                    levmichiconfidence_reg,levhstarts_reg];                
                
allvariablestubs = {'oil','nomoil','gdp','bp1','bp3','bpavg','tr1','tr3','travg','mic','dhsts'};
oilvariablestubs = {'oil','nomoil'};
bpvariablestubs = {'bp1','bp3','bpavg'};
trvariablestubs = {'tr1','tr3','travg'};
allmodelstubs = {'lin','ni3','altni3'};

