function [vhist, betaopt, simopt, exog, restr, inputstring] = modelinput(modelinput,fncn)
%modelinput   
%[vhist, betaopt, simopt, inputstring, exog, restr] = modelinput(modelname,fncn)
%
%Function that takes a string input and convert it into the options for
%running VAR functions: new_estimatemodel, varsim, and other related
%functions.
%
%Input:
%
% *modelname: see below.
% *fncn: fncn is a string, equal to either forecast or impulse. Only
%  necessary for last output - inputstring - which is used in either
%  new_nlfctkvnolead or new_nlimpkv
%
% Modelname is a standardized string input where the first 3 characters
% refer to the core model specification:
% The string is as follows:
% [modeltype][vhist size]_[modeloptions]
%
% Valid entries for modeltype are:
% lin = linear (cntrloptions = -1)
% linerz = linear errors (cntrloptions = -1 for betaopt, for simopt
% cntrloptions = =2)
% mor = Mork Net Increase (cntrloptions = 0, vhist = [])
% ham = Hamiltion net increase (cntrloptions = 0, vhist ~= [])
% gap = Gap model (cntrloptions = 1, vhist ~= [])
% gpm = Gap Mork model (cntrloptions = 7, vhist ~= [])
% net = Net Change model (cntrloptions = 2, vhist ~= [])
% asy = Asymmetric Net Change model (cntrloptions = 3, vhist ~= [])
% lrg = Large model (cntrloptions = 4, vhist = [])
% lpo = Large Positive model (cntrloptions = 5, vhist = [])
% lpn = Large Positive Negative (cntrloptions = 6, vhist = [])
%
% For details on what these options all mean, see the documentation for
% new_estimatemodel, varsim, or your code of choice.
%
% If modeltype is a model that requires a vhist window (ham, gap, net, asy), 
% the string must include a number, in years, in vhist size. If the model
% does not require a vhist window, do not include anything between
% modeltype and _
%
% Model options controls whether the lead, exogenous, restricted, oil, or
% gdp options is selected. For each, the syntax is:
%
% Lead - include an l in modeloptions if you want a lead term for the 
% variable in the 1st column of data array to be included in the 
% regression/simulation for all subsequent variables.
% Cholesky - include a c in modeloptions if you want each successive
% variable to include lead terms for all previous variables (i.e. column 2
% variable has a lead term for column 1 variable, and column 3 variable has
% lead terms for variables in columns 1 and 2).
% Oil - include an o
% Exogenous - include an e
% Restricted - include an r
% Gdp - include a g
%
% **At a minimum, must include a u after the underscore to indicate
%   unrestricted
% **When using lead, still need to specify other options (including u for
%   unrestricted)
%
%Output:
% vhist = size of vhist window in quarters. If no vhist required, vhist is
% empty
% betaopt = a six-element row that is an input for new_estimatemodel
% simopt = a four-element row that is an input for varsim and its variants
% inputstring = the string to put in for OTHEROPTIONS input in functions
%   such as new_nlfcstkvnolead
% exog = 0 or 1 for whether oil is exogenous
% restr = 0 or 1 for whether to use restricted oil for GDP forecast
%
%Example inputs:
%
%'lin_lu' --> Linear unrestricted model with a lead term
%'ham3_lu' --> Hamilton net increase measure with 3-year window w/ a lead
%term
%'ham3_ler' --> Hamilton net increase exogenous restricted with 3-year window
%and lead term
% 
%Written by: Conor Foley
%Last Updated: December 19, 2013
%

if nargout>5 && nargin<2
    msg1 = 'For output 4, must specify input 2: impulse or forecast';
    error('this:fncn',msg1)
end

if nargin<2
    fncn = 'forecast';
end 

model.linerz = -1;
model.lin = -1;
model.ham = 0;
model.mor = 0;
model.gap = 1;
model.net = 2;
model.asy = 3;
model.lrg = 4;
model.lpo = 5;
model.lpn = 6;
model.gpm = 7;

modelname.linerz = 'LinearErrors';
modelname.lin = 'Linear';
modelname.ham = [];
modelname.mor = [];
modelname.gap = 'Gap';
modelname.net = 'NetChange';
modelname.asy = 'AsyNetChange';
modelname.lrg = 'Large';
modelname.lpo = 'LargePositive';
modelname.lpn = 'LargePositiveNegative';
modelname.gpm = 'MorkGap';

model_check = char('lin','ham','mor','gap','net','asy','lrg','lpo','lpn','gpm');
otheropt_check = char('ulcoger');

reqvhist = char('ham','gap','net','asy','gpm');
novhist = char('lin','mor','lrg','lpo','lpn');

modelinput = lower(modelinput);   

if length(modelinput)>6
    if strcmp(modelinput(1:6),'linerz')
        linerzcntrl = 1;
        modelinput = ['lin' modelinput(7:end)];
    else
        linerzcntrl = 0;
    end
else
    linerzcntrl = 0;
end
    
if isempty(intersect(modelinput(1:3),model_check,'rows'))
    error('Fil:ler','Invalid model type')
end

cntrloptions = model.(char(modelinput(1:3)));
inputstring = modelname.(char(modelinput(1:3)));
if linerzcntrl == 1;
    inputstring = 'LinearErrors';
end    

if isempty(intersect(modelinput(1:3),reqvhist,'rows'))
    vhist = [];
    modelinput = modelinput(5:end);
else
    if isnan(str2double(modelinput(4)))
        error('Fil:ler','This model requires you to specify a positive integer for vhist.')
    else
        vhist = 4*str2double(modelinput(4));
        modelinput = modelinput(6:end);
    end
end    

if isempty(modelinput)
    error('Fil:ler','Other options selection invalid (note no selection requires "u" for unrestricted.')
end
    
testval = intersect(modelinput,otheropt_check);
if length(testval) ~= length(modelinput)
    error('Fil:ler','Other options selection invalid (note no selection requires "u" for unrestricted.')
end

incomp = char('erog');
if ~isempty(strfind(modelinput,'u')) && ...
        ~isempty(intersect(modelinput,incomp));
    error('Fil:ler','Incompatible options selected. \n Unrestricted cannot be used with exogenous or restricted.')
end    

if ~isempty(strfind(modelinput,'l')) || ~isempty(strfind(modelinput,'c'))
    otheroptcheck = char('uerog');
    if isempty(intersect(modelinput,otheroptcheck));
        msg1 = 'When selecting lead, still need to select other options.';
        msg2 = '*Note, this may require a u for unrestricted.';
        error('Fil:ler',[msg1 '\n' msg2])
    end
    if ~isempty(strfind(modelinput,'l'))
        leadopt = 0;
        if strcmp(fncn,'forecast')
            inputstring = [inputstring 'Lead'];
        end
    elseif ~isempty(strfind(modelinput,'c'))
        leadopt = 2;
        inputstring = [inputstring 'Cholesky'];
    end
elseif isempty(strfind(modelinput,'l'))
    leadopt = 1;
    if strcmp(fncn,'impulse')
        inputstring = [inputstring 'NoLead'];
    end
end

if ~isempty(strfind(modelinput,'e'))
    exog = 1;
else
    exog = 0;
end

if ~isempty(strfind(modelinput,'r'))
    restr = 1;
else
    restr = 0;
end

if ~isempty(strfind(modelinput,'o'))
    oilopt = 1;
    inputstring = [inputstring 'Oil'];
else
    oilopt = 0;
end

if ~isempty(strfind(modelinput,'g'))
    gdpopt = 1;
    inputstring = [inputstring 'Gdp'];
else
    gdpopt = 0;
end

betaopt = [exog restr oilopt gdpopt leadopt cntrloptions];

if linerzcntrl == 1;
    cntrloptions = -2;
end    

simopt = [restr gdpopt leadopt cntrloptions];

end

