function [cntrb,cumcntrb,frcst_var,cumfrcst_var,cntrb_var,cumcntrb_var] = new_altvardecomp(dataplus,vardatalevel,beta,erz,nlags,hascon,cntrbfromvars,cntrbtovars,ndraws,nstep,montecarlo,stpoint,vhist,options,randseed)
%new_vardecomp 
%Using data (including non-linear terms), betas and errors from 
%new_estimatemodel, this function generates variance decompositions. 
%[cntrb, cumcntrb, frcst_var, cumfrcst_var, cntrb_var, cumcntrb_var] = 
%   new_vardecomp(dataplus,vardatalevel,betasr,ez,nlags,hasconst,
%                 cntrbfromvars,cntrbtovars,ndraws,nstep,montecarlo,
%                 stpoint,vhist,options,randseed)
%
%
%**Inputs:
%DATAPLUS: First columns are VAR variables, columns to the right are any
%non-linear terms calculated in new_estimatemodel
%VARDATALEVEL: Levels of the first nvar columns of LEVELS. Used to estimate
%level of simulated data.
%
%BETA: coefficients estimated by new_estimatemodel, used to calculate
%simulated data.
%
%ERZ: errors from VAR regression from new_estimatemodel
%
%NLAGS: number of lags
%
%HASCON: =1 if a constant was included in regression in new_estimatemodel
%
%CNTRBFROMVARS: Variables you want to see the contribution from. Vector of
%column indeces corresponding to columns for vardatalevel/rows of beta. If
%input is empty array, cntrbfromvars set to 1:nvars. Sheets (3rd dimension)
%of outputs cntrb and cumcntrb correspond to indexes given in 
%cntrbfromvars. CNTRBFROMVARS entries must be column indecies of 
%vardatalevel, and must be unique values.
%
%CNTRBTOVARS: Variables you want to see the contribution to. Vector of
%column indeces corresponding to columns for vardatalevel/rows of beta. If
%input is empty array, cntrbtovars set to 1:nvars. Columns (2nd
%dimension) of outputs cntrb and cumcntrb correspond to indexes given in
%cntrbtovars. CNTRBTOVARS entries must be column indecies of vardatalevel, 
%and must be unique values.
%
%NDRAWS: the size of simulated data sets to generate. Function calculates
%variance on a step-by-step basis over ndraws forecasts.
%
%NSTEP: the number of steps ahead to include in cntrb and cumcntrb
%OPTIONAL  
%
%MONTECARLO: = 1 means that one draws from a normal distribution with same
%mean and variance as the estimated residuals. Otherwise montecarlo = 0
%means that one draws from the empirical distribution of the residuals.
%Default value is 1.
%
%STPOINT: the last observation in dataplus used to calculate the first
% observation of simulated data. For stpoint = 1, the nlag-th observation 
% of dataplus is the last piece of data known before generating forecasts.
%
%VHIST: determines the number of periods over which the window for
% the net increase (or decrease) measure is calculated. Default value is
% empty. If using options Gap, NetChange, or AsyNetChange, must have a
% positive integer for VHIST. If using options Large, LargePositive, or
% LargePositiveNegative, VHIST must be empty. 
% If cntrloptions = 0, there are two possibilites. If VHIST is empty, the 
% Mork measure is used (i.e. positive oil price differences are the 
% non-linear term). If VHIST is not empty, Hamilton's net increase measure
% is used. Note that VHIST = 1 is equivalent to Mork specification.
%
%OPTIONS: A row vector with 4 columns. If one term is specified, all terms
%must also be specified. The columns correpond to:
% 1 - RESTR: =1 if non-oil variables are not a function of linear oil
%       terms. Default value is 0.
% 2 - LEADOPT: =1 if no lead terms are included in any regressions. Default
%       values is 0
% 3 - GDPOPT: =1 if regression for GDP included non-linear values of
%       changes in GDP. Default value is 0.
% 4 - CNTRLOPTIONS: Takes values -1 to 6 corresponding to type of
%       non-linear terms for oil used.
% Here, options is used to ensure coefficients and standard errors are
% organized appropriately. For how options affect regression, more details
% are in code for new_estimatemodel.
%
%RANDSEED: A seed for Matlab's random number generator, using the syntax:
% rand('seed',randseed);
% If no randseed given, calculations done using current seed state.
%
%Output:
%Variance contribution is given as variance with only shocks from 1 
%variable divided by variance with all errors. Variance calculated over
%ndraws simulations.
%
%CNTRB = An nstep x cntrbfromvars x cntrbtovars array. The 1st dimension is
%the n-step ahead forecast variance. The second dimension is the
%contribution from variable X, where X is the corresponding value in
%CNTRBTOVARS. Along the 3rd dimension, is the contribution to variance of
%variable Y, where Y corresponds to value in CNTRBFROMVARS.
%
%CUMCNTRB = An nstep x cntrbfromvars x contrbtovars array. The structure is
%the same as for CNTRB above. CUMCNTRB looks at the variance in levels,
%assuming that the data can be converted to levels by taking the cumulative
%sum over time.
%
%FRCST_VAR = Variance of forecasts, calculated over ndraws forecasts.
%Columns of FRCST_VAR correspond to CNTRBTOVARS.
%
%CUMFRCST_VAR = Variance of cumulative forecasts, calculated over ndraws
%forecasts. Forecasts are converted to "levels" by taking cumulative sum. 
%
%CNTRB_VAR = Variance of forecasts, calculated over ndraws forecasts, when
%there are only innovations from 1 variable. Columns of CNTRB_VAR
%correspond to entries of CNTRBTOVARS. Sheets (3rd dimension) correspond to 
%values of CNTRBFROMVARS.
%
%CUMCNTRB_VAR = Variance of cumulative forecasts, calculated over ndraws 
%forecasts, when there are only innovations from 1 variable. Columns of 
%CUMCNTRB_VAR correspond to entries of CNTRBTOVARS. Sheets (3rd dimension) 
%correspond to values of CNTRBFROMVARS.
%
%See also: varsim   varsim_errfeed
%
%Last Updated by: Conor Foley
%Last Updated on: December 17, 2013


nvars = size(vardatalevel,2);

if isempty(cntrbfromvars)
    cntrbfromvars = 1:nvars;
else
    if ~isvector(cntrbfromvars)
        error('cntrbfromvars must be a vector of indexes corresponding to columns of vardatalevel.')
    end

    if length(cntrbfromvars)~=length(unique(cntrbfromvars))
        error('cntrbfromvars does not contain all unique values.')
    end

    if intersect(cntrbfromvars,1:nvars,'stable') ~= cntrbfromvars
        error('cntrbfromvars has an index value not equal to a column of vardatalevel')
    end
end

if isempty(cntrbtovars)
    cntrbtovars = 1:nvars;
else
    if ~isvector(cntrbtovars) || ~isnumeric(cntrbtovars)
        error('cntrbtovars must be a numeric vector of indexes corresponding to columns of vardatalevel.')
    end

    if length(cntrbtovars)~=length(unique(cntrbtovars))
        error('cntrbtovars does not contain all unique values.')
    end

    if intersect(cntrbtovars,1:nvars,'stable') ~= cntrbtovars
        error('cntrbtovars has an index value not equal to a column of vardatalevel')
    end
end
    
if nargin>14
    rand('seed',randseed);
end

feederror = NaN(nstep,nvars,ndraws);
for zy = 1:nstep;
    rr1 = rand(ndraws,1);
    tmp1 = reshape(erz(ceil(size(erz,1)*rr1),:)',[1 nvars ndraws]);
    feederror(zy,:,:) = tmp1;
end

[frcst_diff] = varsim_errfeed(dataplus,vardatalevel,beta,erz,nlags,hascon,[1:nvars],feederror,ndraws,nstep,montecarlo,stpoint,vhist,options);
cumfrcst_diff = cumsum(frcst_diff(nlags+1:end,:,:),1);
frcst_var = var(frcst_diff(nlags+1:end,:,:),0,3);
cumfrcst_var = var(cumfrcst_diff,0,3);

cntrb = NaN(nstep,length(cntrbtovars),length(cntrbfromvars));
cumcntrb = NaN(nstep,length(cntrbtovars),length(cntrbfromvars));

if nargout > 4
    cntrb_var = NaN(nstep,length(cntrbtovars),length(cntrbfromvars));
    cumcntrb_var = NaN(nstep,length(cntrbtovars),length(cntrbfromvars));
end

tmpbetaopt = zeros(1,6);
tmpbetaopt([2 3 5 6]) = options;

[~,~,~,~,~,indvar] = new_estimatemodel(dataplus(:,1:nvars),vardatalevel,nlags,hascon,[],vhist,tmpbetaopt);

actualerz = dataplus(nlags+1:end,1:nvars)-indvar*beta';
stpt_erz = actualerz(stpoint+(0:nstep-1),:);

for ifvar = 1:length(cntrbfromvars)
    
    fromvar = cntrbfromvars(ifvar);
    
    feederror_test = repmat(stpt_erz,[1 1 ndraws]);
    feederror_test(:,fromvar,:) = feederror(:,fromvar,:);
    
    [testvar_diff] = varsim_errfeed(dataplus,vardatalevel,beta,erz,nlags,hascon,[1:nvars],feederror_test,ndraws,nstep,montecarlo,stpoint,vhist,options);
    cumtestvar_diff = cumsum(testvar_diff(nlags+1:end,:,:),1);
    testvar_var = var(testvar_diff(nlags+1:end,:,:),0,3);
    cumtestvar_var = var(cumtestvar_diff,0,3);    

    cntrb(:,:,ifvar) = testvar_var(:,cntrbtovars)./frcst_var(:,cntrbtovars);
    cumcntrb(:,:,ifvar) = cumtestvar_var(:,cntrbtovars)./cumfrcst_var(:,cntrbtovars);
    
    if nargout > 4
        cntrb_var(:,:,ifvar) = testvar_var(:,cntrbtovars);
        cumcntrb_var(:,:,ifvar) = cumtestvar_var(:,cntrbtovars);
    end
    
end

if nargout>2
    frcst_var = frcst_var(:,cntrbtovars);
    cumfrcst_var = cumfrcst_var(:,cntrbtovars);
end
 

end