function [simdata,levelysimdata,simerz] = varsim(levels,vardatalevel,betasr,ez,nlags,hasconst,ndraws,nstep,montecarlo,stpoint,vhist,options)
%varsim Using data (including non-linear terms), betas and errors from new_estimatemodel generates simulated data.
%[simdata, levelysimdata, simerz] = varsim(levels,vardatalevel,betasr,ez,nlags,hasconst,ndraws,nstep,montecarlo,stpoint,vhist,options)
%
%**Inputs:
%LEVELS: First columns are VAR variables, columns to the right are any
%non-linear terms calculated in new_estimatemodel
%VARDATALEVEL: Levels of the first nvar columns of LEVELS. Used to estimate
%level of simulated data.
%BETASR: coefficients estimated by new_estimatemodel, used to calculate
%simulated data
%EZ: errors from VAR regression from new_estimatemodel
%NLAGS: number of lags
%HASCONST: =1 if a constant was included in regression in new_estimatemodel
%NDRAWS: the number of simulations to do per history. Take the average over
%the number of simulations to arrive at simdata and levelysimdata.
%NSTEP: the number of steps to have for the impulse reponses/forecast/size
%of simulated time-series.
%OPTIONAL  
%MONTECARLO: = 1 means that one draws from a normal distribution with same
%mean and variance as the estimated residuals. Otherwise montecarlo = 0
%means that one draws from the empirical distribution of the residuals.
%Default value is 1.
%STPOINT: the last observation in VARDATA used to calculate the first
% observation of simulated data.
%VHIST: determines the number of periods over which the window for
% the net increase (or decrease) measure is calculated. Default value is
% empty. If using options Gap, NetChange, or AsyNetChange, must have a
% positive integer for VHIST. If using options Large, LargePositive, or
% LargePositiveNegative, VHIST must be empty. Changing VHIST changes action
% of default option for non-linear oil term (accepts empty or positive
% integer).
%OPTIONS: A row vector with 4 columns. If one term is specified, all terms
%must also be specified. The columns correpond to:
% 1 - RESTR: =1 if non-oil variables are not a function of linear oil
%       terms. Default value is 0.
% 2 - LEADOPT: =1 if no lead terms are included in any regressions. Default
%       values is 0
% 3 - GDPOPT: =1 if regression for GDP included non-linear values of
%       changes in GDP. Default value is 0.
% 4 - CNTRLOPTIONS: Takes values -1 to 6 corresponding to type of
%       non-linear terms for oil used.
% Here, options is used to ensure coefficients and standard errors are
% organized appropriately. For how options affect regression, more details
% are in code for new_estimatemodel.
%
%Output:
%SIMDATA: An nvar x nstep+nlag x ndraws array of simulated data, with each
%nvar x nstep+nlag array being a seperate simulation
%LEVELYSIMDATA: Levels of SIMDATA
%SIMERZ: The 3-dimensional array of error terms used in calculating all
%simulations (nstep x nvars x ndraws).
%
%Last Updated by: Conor Foley
%Last Updated on: January 15, 2013

%% Feed in options/default values
if or(~exist('options','var'),isempty(options));
    restr = 0;
    leadopt = 0;
    gdpopt = 0;
    cntrloptions = 0;
else
    restr = options(1,1);
    gdpopt = options(1,2);
    leadopt = options(1,3);
    cntrloptions = options(1,4);
    
end

if or(~exist('montecarlo','var'),isempty(montecarlo));
    montecarlo = 1; %default value is 1
end

if ~exist('vhist','var')
    vhist = [];
end

%%
%% Set number of lead and linear, and non-linear terms

nvars = size(betasr,1);

numnl = size(levels,2) - nvars;

if cntrloptions == 3 || cntrloptions == 6 || cntrloptions == 7;
    %AsyNetChange or LargePositiveNegtiave or MorkGap
    numnloil = 2;
else
    numnloil = 1;
end   

if leadopt == 1; %NoLead options on (impulse response), or standard forecast model
    nlead = 0;
elseif leadopt == 2; %RollLead option
    nlead = nvars-1+numnl-gdpopt;
elseif numnloil==2; 
    %Options AsyNetChange or LargePositiveNegative or MorkGap
    nlead = 3-restr; %number of non-linear terms plus oil (if included)
elseif cntrloptions < 0 %Option Linear or LinearErrors
    nlead = 1;
else
    nlead = 2-restr;
end

%%
%% Set censorthreshold, if necessary
if cntrloptions == 4 || cntrloptions == 5 || cntrloptions == 6; 
    %Large, LargePositive, LargePositiveNegative
    censorthreshold = std(levels(:,1));
end
%%
%% Break betasr into constituent parts

%break out constants
if hasconst == 1;
    bc = betasr(:,hasconst); 
else
    bc = zeros(size(betasr,1),1);
end

%breat out coefficients on lead terms
if leadopt == 1; %No Leads
    gamval = zeros(nvars,nvars);
    gamval_nloil = zeros(nvars,2);
else
    gamval = betasr(:,hasconst+(nvars+numnl)*nlags+1:end);
    
    if restr == 1; %set oil lead coefficients to zero if used restricted option
        gamval = [zeros(nvars,1) gamval];
    end
    
    if numnl > 0
        gamval_nloil = gamval(:,1+(1:numnl));
        gamval = [gamval(:,1) gamval(:,1+numnl+1:end)];
    else
        gamval_nloil = zeros(nvars,2);
    end
    
    if size(gamval,2)~=nvars
        gamval = [gamval zeros(nvars,nvars-size(gamval,2))];
    end
    
end

%break out coefficients for linear VAR terms
bslp = betasr(:,hasconst+(1:nvars*nlags));

%break out coefficients for non-linear gdp
if gdpopt == 1;
    dumpartgdp = betasr(:,hasconst+(nvars*nlags)+(numnl:numnl:numnl*nlags));
end

%break out coefficients for negative non-linear oil terms, if any 
if numnloil == 2;
    %AsyNetChange or LargePositiveNegative or MorkGap
    dumparttwo = betasr(:,hasconst+(nvars*nlags)+(2:numnl:numnl*nlags));
end

%break out coefficients for non-linear oil terms
if cntrloptions < 0; %for Linear or LinearErrors
    dumpart = zeros(nvars,nlags);
else
    dumpart = betasr(:,hasconst+(nvars*nlags)+(1:numnl:numnl*nlags));
end


%%
%% Use errors to make Cholesky

if montecarlo == 1;
    signew = cov(ez);
    facsignew = chol(signew)';
    factran = facsignew';
end

%%
%% Adjust values for ease of linear model
if cntrloptions == -1;
    ndraws = 1;
    ez = 0*ez;
    if montecarlo == 1
        factran = 0*facsignew;
    end
end

%%
%% Prepare initial data for running simulations

%preallocate main matrices we will fill out with simulated data
ydata = zeros(nstep+nlags, nvars);
levelydata=ydata;
if cntrloptions < 0; %for Linear or LinearErrors
    netydata = zeros(nstep+nlags,2);
else    
    netydata = zeros(nstep+nlags, numnl-gdpopt);
end

%fill in first nlag rows of above matricies with appropriate initial data
%from vardata
if isnan(stpoint);
    ydata(1:nlags,1:nvars)=0;
    levelydata(1:nlags,1:nvars)=0;
    netydata(1:nlags,1)=0;
    stmax=0;
    stmin = 0;
elseif stpoint == -1;
    ydata(1:nlags,1:nvars) = repmat(mean(levels(:,1:nvars)),nlags,1);
    levelydata(1:nlags,1:nvars) = 0;
    netydata(1:nlags,1) =  0;
    stmax = 0;
    stmin = 0;
else
    %use the original values as starting values
    ydata(1:nlags,1:nvars) = levels(stpoint+(0:nlags-1),1:nvars); 
    levelydata(1:nlags,1:nvars) = vardatalevel(1+stpoint+(0:nlags-1),1:nvars);
    if cntrloptions < 0 %Linear option - leave netydata as all zeros
        netydata(1:nlags,1) = zeros(nlags,1);
    else %otherwise, use non-linear oil terms in levels
        netydata(1:nlags,1:(numnl-gdpopt)) = levels(stpoint+(0:nlags-1),nvars+(1:numnl-gdpopt));
    end
    
    if gdpopt == 1;
        netgdpdata(1:nlags,1) = levels(stpoint+(0:nlags-1),nvars+numnl);
    end
        
    %levels includes vardata with any non-linear terms used in estimating,
    %coefficients with non-linear term for GDP always being the last column 
    %(if included)
    
    if ~isempty(vhist)
        stmax = max(vardatalevel(max(1,1+stpoint+nlags-vhist):max(stpoint+nlags,1)));
    else
        stmax = [];
    end
    
    if ~isempty(intersect(cntrloptions,[2 3 4 5 6]));
        %option NetChange, AsyNetChange, Large, LargePositive, or
        %LargePositiveNegative
        stmin = min(vardatalevel(max(1,1+stpoint+nlags-vhist):max(stpoint+nlags,1)));
    else
        stmin = [];
    end
    
end

%set up lagged values - will be carried around in our calculations

%reverse order of variables (to count backwards instead of forwards) and
%transpose into a row (instead of a column)

%netzlag = reshape(netydata(nlags:-1:1,1:nvars)',1,nlags*nvars)
firstnetzlag = netydata(nlags:-1:1,1)';

if numnloil==2;
    %options AsyNetChange or LargePositiveNegative or MorkGap
    firstnetzlagtwo = netydata(nlags:-1:1,2)';
end

if gdpopt == 1;
    firstnetzlaggdp = netgdpdata(nlags:-1:1,1)';
end    

zlag = reshape(ydata(nlags:-1:1,1:nvars)',1,nlags*nvars);

largestlastvhist = stmax;
smallestlastvhist = stmin;

%use repmat to expand sheets into 3-dimensional arrays to allow us to
%compute all the simulations simultaneously

ydata = repmat(ydata,[1 1 ndraws]);
levelydata = repmat(levelydata, [1 1 ndraws]);
netydata = repmat(netydata, [1 1 ndraws]);
firstnetzlag = repmat(firstnetzlag, [1 1 ndraws]);
zlag = repmat(zlag, [1 1 ndraws]);
stmax = repmat(stmax, [1 1 ndraws]);
stmin = repmat(stmin, [1 1 ndraws]);

if numnloil==2;
    %option AsyNetChange or LargePositiveNegative or MorkGap
    firstnetzlagtwo = repmat(firstnetzlagtwo, [1 1 ndraws]);
end    

if gdpopt == 1;
    netgdpdata = repmat(netgdpdata, [1 1 ndraws]);
    firstnetzlaggdp = repmat(firstnetzlaggdp, [1 1 ndraws]);
end    

largestlastvhist = stmax;
smallestlastvhist = stmin;

if cntrloptions==2 || cntrloptions ==3;
    %NetChange or AsyNetChange
    netpos = zeros(1, 1, ndraws);
    negnet = zeros(1, 1, ndraws);
end

%%
%% Generate a 3-dimensional array of errors
erz = NaN(nstep,nvars,ndraws);

%May need to use random number generator seed for debugging
%rng(5);

if montecarlo ==1;
    rr1 = randn(nstep,nvars,ndraws);
    
    for zx = 1:nstep;
        for zp = 1:nvars;
            erz(zx,zp,:) = sum(bsxfun(@times,rr1(zx,:,:),factran(:,zp)'),2);
        end
    end
%use bsxfun to multiply the appropriate entry in facsignew (transposed to
%allow for element-by-element operations) by a random shock. Each sequence
%of the loop over zx fills in a single row of all sheets (the 3rd 
%dimension, ndraws) of erz. Each loop over zp fills in a column of the 
%zx-th row (for each variable).

%erz is an nstep x nvar x ndraws matrix    

else
    for zx = 1:nstep;
        rr1 = rand(ndraws,1);
        tmp1 = reshape(ez(ceil(size(ez,1)*rr1),:)',[1 nvars ndraws]);
        erz(zx,:,:) = tmp1;
 %use ceil to generate random numbers bounded between 1 and number of residuals
 
 %make a ndraw x nvar matrix of random draws of rows of uz, then reshape 
 %into a 1 x nvar x ndraw matrix, and make this the zx-th row (1st 
 %dimension)of erz
 
 %erz becomes an nstep x nvar x ndraws matrix
 %this is equivalent to random sampling with replacement from residuals uz
    
    end  
end
%%
%% Run simulations

tmp = zeros([1 nvars ndraws]);

%generate simulated data without shocks by using vectorized inputs and
%looping over nsteps and nvars. Each loop over nstep fills in a new
%observation of simulated data (a row in the output matrix), while each 
%loop over nvars fills in a new column for the intermediate variable we use
%to build the row currently being worked on. Loop also keeps track of
%changes in non-linear terms, if any, and organizes appropriately for next
%step.
for zx = nlags+(1:nstep);
    
    %run calculation for all variables on lags of all other vars in vardata
    %and lags of non-linear oil
    for zp = 1:nvars;
        tmp(1,zp,:) = bc(zp,1) + sum(bsxfun(@times,bslp(zp,:),zlag),2) + sum(bsxfun(@times,dumpart(zp,:),firstnetzlag),2) + erz(zx-nlags,zp,:);
        
        if numnloil==2; 
            %AsyNetChange or LargePositiveNegative or MorkGap
            tmp(1,zp,:) = tmp(1,zp,:) + sum(bsxfun(@times,firstnetzlagtwo,dumparttwo(zp,:)),2);
        end
        
    end
    
    %fill in new row of simulated data for all variables
    %NOTE: We are only done with oil. Still need to add on lead terms for
    %non-oil variables and/or non-linear GDP.
    ydata(zx,:,:) = tmp(1,:,:);
    
    %Continue filling in next row by now including effect of lead terms
    
    for zp=1:nvars;
        
        if leadopt == 0; %add lead terms just for oil
            ydata(zx,zp,:) =  ydata(zx,zp,:) + gamval(zp,1)*ydata(zx,1,:);
            
        elseif leadopt == 2; %for Cholesky, add leads of preceding variables
                ydata(zx,zp,:) = ydata(zx,zp,:) + sum(bsxfun(@times,gamval(zp,:),ydata(zx,:,:)),2);
        end
            
    end
        
    %advance netydata to latest period
    if ~isempty(vhist);
        netydata(zx,1,:) = max(levelydata(zx-1,1,:)+ydata(zx,1,:)-largestlastvhist,0); %Hamilton Net Increase
        %fill in the new entry for oil with the difference between the new
        %level (calculated as previous level + change) minus the recent
        %high, or zero, whichever is larger
        if cntrloptions == 1; %option Gap
            netydata(zx,1,:) = levelydata(zx-1,1,:)+ydata(zx,1,:) - largestlastvhist;
            %fill in the new entry for oil with the difference between the
            %new level (calculated as the previous level + change) minus
            %the recent high
        elseif cntrloptions == 7; %option MorkGap
            netydata(zx,1,:) = levelydata(zx-1,1,:)+ydata(zx,1,:) - largestlastvhist;
            netydata(zx,2,:) = (netydata(zx,1,:)>0).*netydata(zx,1,:);
            %same as gap above, except also include a second term for only
            %positive gap measures
        elseif cntrloptions == 2 || cntrloptions == 3;
            netpos =  max(levelydata(zx-1,1,:)+ydata(zx,1,:) - largestlastvhist,0);
            %a series of 1x1 sheets, equal to the the difference between
            %the new level and the recent high, or zero if that is negative
            netneg =  min(levelydata(zx-1,1,:)+ydata(zx,1,:) - smallestlastvhist,0);
            %a series of 1x1 sheets, equal to the the difference between
            %the new level and the recent low, or zero if that is positive
            if cntrloptions == 2; %option NetChange
                netydata(zx,1,:) = netpos + netneg;
                %add to get new entries in netydata column 1
            elseif cntrloptions == 3; %option AsyNetChange
                netydata(zx,1:2,:) =[netpos  netneg];
                %carry along each as a separate column
            end
        end 
    else %if vhist is empty
        netydata(zx,1,:)= max(ydata(zx,1,:),0); %Mork measure
              %netydata = ydata if ydata>0; otherwise =0
        if cntrloptions == 4 %option Large
            netydata(zx,1,:) = bsxfun(@times,ydata(zx,1,:),(abs(ydata(zx,1,:))> censorthreshold) ) ;
            %netydata = ydata if magnitude in change of price > standard 
            %deviation of price changes, otherwise = 0
        elseif cntrloptions == 5 %option LargePositive
            netydata(zx,1,:) = bsxfun(@times,ydata(zx,1,:),(ydata(zx,1,:)> censorthreshold) );
            %netydata = ydata if price change is positive AND is greater
            %than the standard deviation of price changes, otherwise = zero
        elseif cntrloptions == 6 %option LargePositiveNegative
            netydata(zx,1,:) = bsxfun(@times,ydata(zx,1,:),(ydata(zx,1,:)> censorthreshold) );
            netydata(zx,2,:) = bsxfun(@times,ydata(zx,1,:),(ydata(zx,1,:)< -censorthreshold) );
            %first column = positive price changes of more than 1 standard dev
            %second column = negative changes of more than 1 standard dev
        end
    end
    
    %finish filling in new row by adding to earlier estimate (based on lags 
    %of all linear terms and non-linear oil terms, plus leads of linear 
    %terms) the influence of any lead terms for non-linear values (and for 
    %gdp, the influence of non-linear gdp term if using gdpopt)
    for zp=1:nvars;
        
        if or(leadopt == 0, leadopt == 2); %add lead terms
            ydata(zx,zp,:) =  ydata(zx,zp,:) + gamval_nloil(zp,1)*netydata(zx,1,:);

            if numnloil==2;
                ydata(zx,zp,:) = ydata(zx,zp,:) + gamval_nloil(zp,2)*netydata(zx,2,:);
            end
            
        end
        
        if and(zp == nvars,gdpopt == 1); %including non-linear GDP, only for GDP, if selected
            ydata(zx,zp,:) = ydata(zx,zp,:) + sum(bsxfun(@times,dumpartgdp(zp,:),firstnetzlaggdp),2);
        end
    
    end
        
    %advance other intermediate variables to latest time period using 
    %newest simulated data, and dropping column(s) that represent lags now
    %unnecessary for next estimation
    zlag = [ydata(zx,:,:) zlag(1,1:(nlags-1)*nvars,:)];
    firstnetzlag = [netydata(zx,1,:) firstnetzlag(1,1:(nlags-1),:)];
    levelydata(zx,:,:) = levelydata(zx-1,:,:) + ydata(zx,:,:);
    
    %if using firstnetzlagtwo, also advance that variable
    
    if numnloil==2; 
        %AsyNetChange or LargePositiveNegative or MorkGap
        firstnetzlagtwo = [netydata(zx,2,:) firstnetzlagtwo(1,1:(nlags-1),:)];
    end
    
    %if using gdpopt, also advance netgdpdata and firstnetzlaggdp
    if gdpopt == 1;
        netgdpdata(zx,1,:)= max(ydata(zx,nvars,:),0);
        firstnetzlaggdp = [netgdpdata(zx,1,:) firstnetzlaggdp(1,1:(nlags-1),:)];
    end
    
    if ~isempty(vhist);
        if zx<(vhist);
            newmax = repmat(max(vardatalevel(max(stpoint+zx-vhist+1,1):stpoint+nlags,1),[],1),[1 1 size(levelydata,3)]);
            largestlastvhist = max(max(levelydata(max(1,zx-vhist-1):zx,1,:),[],1),newmax);
            if isempty(stmin)
                smallestlastvhist = [];
            else
                newmin = repmat(min(vardatalevel(max(stpoint+zx-vhist+1,1):stpoint+nlags,1),[],1),[1 1 size(levelydata,3)]);
                smallestlastvhist = min(min(levelydata(max(1,zx-vhist-1):zx,1,:),[],1),newmin);
            end
                %matlab doesn't allow you to take minimum over an array
                %relative to the empty set (which also cannot be repmatted)
                %this controls for that behavior

        else
            largestlastvhist = max(levelydata(zx-vhist+1:zx,1,:),[],1);
            smallestlastvhist = min(levelydata(zx-vhist+1:zx,1,:),[],1);
        end    
    end
end

%store results for full set of simulations to be used later in comparison
%to estimates using shocks

simdata = ydata;
levelysimdata= levelydata;

if nargout>2
    simerz = erz;
end


%%
%% End of function

end

